!***********************************************************************************************************************************
!**    S U B R O U T I N E    I N I T I A L    W A T E R    L E V E L                                                            **
!**********************************************************************************************************************************

subroutine initial_water_level

use main
USE GLOBAL;     USE NAMESC; USE GEOMC;  USE LOGICC; USE PREC;  USE SURFHE;  USE KINETIC; USE SHADEC; USE EDDY
  USE STRUCTURES; USE TRANS;  USE TVDC;   USE SELWC;  USE GDAYC; USE SCREENC; USE TDGAS;   USE RSTART
  use macrophytec; use porosityc; use zooplanktonc 
  use initialvelocity
  EXTERNAL RESTART_OUTPUT
  integer :: jbu,jbd, jjw
  real :: qgate,wsup,brlen,wlslope,dist, wldiff
  
  loop_branch=.false.
    
! estimating initial flows in each segment
     qssi=0.0
            
! first considering specified flows: upstream inflows, tributaries, distributed tribs, structural withdrawals and withdrawals    
    DO JW=1,NWB
      KT = KTWB(JW)
      DO JB=BS(JW),BE(JW)
        IU = CUS(JB)
        ID = DS(JB)
        do i=iu,id
          if(i == iu .and. UP_FLOW(JB))then
             qssi(i)=qin(jb)+qssi(i)
          end if
! if downstream of structure          
          !IF (i == iu .and. UHS(JB) < 0) then
          IF (i == iu .and. DAM_INFLOW(JB)) then    ! cb 4/27/2011
            DO JjW=1,nwb
              DO JJB=BS(JjW),be(jjw)
                if(ds(jjb) == abs(uhs(jb)))then
                  DO JS=1,NSTR(JjB)              
                    qssi(i)=qssi(i)+qstr(js,jjb)
                  end do
                end if
              end do
            end do
          end if

          IF (TRIBUTARIES) THEN
            DO JT=1,ntr
              if(itr(jt) == i)qssi(i)=qssi(i)+qtr(jt)
            END DO
          END IF
          IF (DIST_TRIBS(JB)) THEN
            qssi(i)=qssi(i)+qdtr(jb)/real(id-iu+1)    ! since initial wl unknown, distributing flow evenly btw. segs.              
          END IF
          IF (WITHDRAWALS) THEN
            DO JWD=1,NWD
              if(iwd(jwd) == i)qssi(i)=qssi(i)-qwd(jwd)            
            END DO
          END IF
          if(i == id)then
            DO JS=1,NSTR(JB)              
              qssi(i)=qssi(i)-qstr(js,jb)
            END DO
          end if
        end do
      END DO
    END DO
    
! including flows within branch upsteam of segment
    DO JW=1,NWB
      KT = KTWB(JW)
      DO JB=BS(JW),BE(JW)
        IU = CUS(JB)
        ID = DS(JB)
        do i=iu+1,id
          qssi(i)=qssi(i)+qssi(i-1)
        end do
      end do
    end do

    DO JW=1,NWB
      KT = KTWB(JW)
      DO JB=BS(JW),BE(JW)
        IU = CUS(JB)
        ID = DS(JB)
        do i=iu,id          
! determining if segment is downstream internal head boundary of another *upstream* branch, and adding flow to segment and segments downstream
          DO JjW=1,nwb
            DO JJB=BS(JjW),be(jjw)
              if(dhs(jjb) == i)then
                do ii=i,id
                  qssi(ii)=qssi(ii)+qssi(ds(jjb))
                end do
              end if
            end do
          end do                    
! determining if segment is downstream of spillway below another branch
          do JS=1,NSP
            if(ESP(JS) < el(2,i))then  ! disregarding if crest above grid
              if(i == IDSP(JS))then
                do ii=i,id
                  qssi(ii)=qssi(ii)+qssi(IUSP(JS))
                end do
              end if
            end if
          end do
! determining if segment is downstream of gate below another branch
          do Jg=1,Ngt
            if(Egt(Jg) < el(2,i))then  ! disregarding if crest above grid
              if(i == IDgt(Jg))then
                if(dyngtc(jg) == '    FLOW')then
                  qgate = BGT(JG)
                  do ii=i,id
                    qssi(ii)=qssi(ii)+qgate
                  end do
                else
                  do ii=i,id
                    qssi(ii)=qssi(ii)+qssi(IUgt(Jg))
                  end do
                end if
              end if
            end if
          end do                    
        end do
      END DO
    END DO

! determining if branch is a secondary branch that loops around an island with internal head boundaries at upstream and downstream BC
! attached to a single branch
    DO JW=1,NWB
      KT = KTWB(JW)
      DO JB=BS(JW),BE(JW)
        if(UH_INTERNAL(JB) .and. DH_INTERNAL(JB))then
          DO JjW=1,nwb
              DO JJB=BS(JjW),be(jjw)
                if(uhs(jb) > us(jjb) .and. uhs(jb) < ds(jjb))then
                  jbu=jjb
                end if
              end do
          end do
          DO JjW=1,nwb
              DO JJB=BS(JjW),be(jjw)
                if(dhs(jb) > us(jjb) .and. dhs(j) < ds(jjb))then
                  jbd=jjb
                end if
              end do
          end do
          loop_branch(jb) = jbu == jbd
        end if
      end do
    end do
              
              
! given estimated flows for each segment, estimating WL with normal depth equation
    DO JW=1,NWB
      KT = KTWB(JW)
      DO JB=BS(JW),BE(JW)
! only considering branches with slopes > 0      
        IF (SLOPE(JB) > 0.0 .and. .not. loop_branch(jb))then
          IU = CUS(JB)
          ID = DS(JB)    
          do i=iu,id
            call normal_depth(qssi(i))
          end do
        end if
      end do
    end do
    
! smoothing water surface elevations, first within branches
    DO JW=1,NWB
      KT = KTWB(JW)
      DO JB=BS(JW),BE(JW)
        IF (SLOPE(JB) > 0.0 .and. .not. loop_branch(jb))then
          IU = CUS(JB)
          ID = DS(JB)
          do i=id-1,iu,-1
            if(elws(i+1) > elws(i))then
              elws(i) = elws(i+1)
            end if          
          end do
        end if
      end do
    end do

! smoothing water surface elevations at internal head boundaries
    DO JW=1,NWB
      KT = KTWB(JW)
      DO JB=Bs(JW),Be(JW)
        IF (SLOPE(JB) > 0.0 .and. .not. loop_branch(jb) .and. DH_INTERNAL(JB))then
          IU = CUS(JB)
          ID = DS(JB)
          if(elws(dhs(jb)) > elws(id))then
            elws(id)=elws(dhs(jb))
            do i=id-1,iu,-1
              if(elws(i+1) > elws(i))then
                elws(i) = elws(i+1)
              end if   
            end do
          end if
        end if
      end do
    end do

! if spillway or gate at downstream end of branch, making sure water level is above crest elevation
    DO JW=1,NWB
      KT = KTWB(JW)
      DO JB=BS(JW),BE(JW)
        IF (SLOPE(JB) > 0.0 .and. .not. loop_branch(jb) .and. nstr(jb) == 0)then
          IU = CUS(JB)
          ID = DS(JB)
! spillways          
          do JS=1,NSP
            if(ESP(JS) < el(2,i))then  ! disregarding if crest above grid
              if(id == IuSP(JS))then
                 wsup=esp(js)+(qssi(id)/A1SP(JS))**(1.0/b1sp(js))  ! estimating upstream ws elev
                 if(elws(id) < wsup)then
                   if(idsp(js) .ne. 0)then    ! cb 8/10/10
                   if(elws(idsp(js)) > wsup) wsup = elws(idsp(js))   ! checking to see if downstream ws elevation isn't already 'high'
                   elws(id)=wsup
                   end if                     ! cb 8/10/10
                   do i=id-1,iu,-1
                     if(elws(i+1) > elws(i))then
                       elws(i) = elws(i+1)
                     end if   
                   end do
                 end if
               end if
            end if
          end do
! gates
          do Jg=1,Ngt
            if(Egt(Jg) < el(2,i) .and. dyngtc(jg) .ne. '    FLOW')then  ! disregarding if crest above grid
              if(id == Iugt(Jg))then
                if(dyngtc(jg) == '       B')then
                  wsup=egt(jg)+(qssi(id)/(A1gt(Jg)*BGT(JG)**g1gt(jg)))**(1.0/b1gt(jg))
                end if
                if(dyngtc(jg) == '     ZGT')then
                  wsup=egt(jg)+(qssi(id)/A1gt(Jg))**(1.0/b1gt(jg))
                end if
                if(elws(id) < wsup)then
                   if(elws(idsp(js)) > wsup) wsup = elws(idsp(js))   ! checking to see if downstream ws elevation isn't already 'high'
                   elws(id)=wsup
                   do i=id-1,iu,-1
                     if(elws(i+1) > elws(i))then
                       elws(i) = elws(i+1)
                     end if   
                   end do
                end if
              end if
            end if
          end do
        end if
      end do
    end do

! smoothing water level around loop branches
    DO JW=1,NWB
      KT = KTWB(JW)
      DO JB=BS(JW),BE(JW)
        IU = CUS(JB)
        ID = DS(JB)
        IF (loop_branch(jb))then
          wldiff=elws(uhs(jb))-elws(uhs(jb))          
          brlen=0.0
          do i=iu,id
            brlen=brlen+dlx(i)
          end do
          wlslope=wldiff/brlen
          dist=dlx(iu)/2.0
          elws(iu)=elws(uhs(jb))-wlslope*dist
          do i=iu+1,id
            dist=dist+(dlx(i-1)+dlx(i))/2.0
            elws(iu)=elws(uhs(jb))-wlslope*dist
          end do
        end if
      end do
    end do

    return
    end subroutine initial_water_level

!***********************************************************************************************************************************
!**        S U B R O U T I N E    N O R M A L    D E P T H                                                                        **
!***********************************************************************************************************************************

      subroutine normal_depth(flow)
      
      USE GLOBAL; USE GEOMC
                  
      integer, PARAMETER ::JMAX=40     
      real :: x1, x2, flow, funcval1, funcval2, xacc, fmid, func1, rtbis, dx, xmid     
      integer :: jj,j  
                  
! first, bracketing root
      x1=0.001
      x2=1.0
      call mannings_eqn(flow,x1,funcval1)
      call mannings_eqn(flow,x2,funcval2)
                  
      do jj=1,jmax
        if(funcval1*funcval2 > 0.0)then
          if(abs(funcval1)  < abs(funcval2))then
            x1=x1/2.0
            call mannings_eqn(flow,x1,funcval1)            
          else
            x2=x2+1.5*(x2-x1)
            call mannings_eqn(flow,x2,funcval2)
          end if                                          
        else
          exit
        end if      
      end do                 
      
! finding root by bisection      
      xacc=0.01
      call mannings_eqn(flow,x2,fmid)
      call mannings_eqn(flow,x1,func1)      
  !    if(func1*fmid.ge.0.) pause 'root must be bracketed in rtbis'
      if(func1.lt.0.)then
        rtbis=x1
        dx=x2-x1
      else
        rtbis=x2
        dx=x1-x2
      endif
      do j=1,JMAX
        dx=dx*.5
        xmid=rtbis+dx
        call mannings_eqn(flow,xmid,fmid)        
        if(fmid.le.0.)rtbis=xmid
        if(abs(dx).lt.xacc .or. fmid.eq.0.)then          
          elws(i)=rtbis+el(kb(i)-1,i)
          return
        end if
      end do
  !    pause 'too many bisections in rtbis'
      END subroutine normal_depth

      
!***********************************************************************************************************************************
!**        S U B R O U T I N E    M A N N I N G S    E Q U A T I O N                                                              **
!***********************************************************************************************************************************

      subroutine mannings_eqn(flow,depth,funcvalue)
      
      USE GLOBAL; USE GEOMC; use eddy; use logicc
      
      real :: wsurf, depth, flow, xarea, wper, hrad,fmann, funcvalue

 !     wsurf=el(kb(i)-1,i)+depth
      wsurf=el(kb(i)+1,i)+depth   ! cb 7/7/10
      call xsectional_area(wsurf,xarea)
      wper=b(kti(i),i)+2.0*depth
      hrad=xarea/wper
      if(mannings_n(jw))then
        fmann=fric(i)
      else
        fmann=hrad**0.166666667/fric(i)
      end if
      funcvalue=flow-xarea*hrad**0.6667*slope(jb)**0.5/fmann
      
      return
      end subroutine mannings_eqn
      
!***********************************************************************************************************************************
!**        S U B R O U T I N E    C R O S S    S E C T I O N A L    A R E A                                                       **
!***********************************************************************************************************************************

      subroutine xsectional_area(wsurf,xarea)
      
      USE GLOBAL; USE GEOMC; use main; use initialvelocity
      
      real :: kttop, xarea, wsurf

 !     KTtop = 2
      do k=2,kmx-1
        if(el(k,i) < wsurf)then
          kttop=k-1
          exit
        end if
        kttop=k     ! cb 8/10/10
      end do
 !     DO WHILE (EL(KTtop,I) > wsurf)
 !        KTtop = KTtop+1
 !     END DO            
      xarea=(wsurf-el(kttop+1,i)) * bsave(kttop,i)
      DO K=kttop+1,kbi(i)
         xarea = xarea+Bsave(k,i)*h(k,jw)
      END DO      
      
      return
      end  subroutine xsectional_area
      
      
!***********************************************************************************************************************************
!**        S U B R O U T I N E    I N I T I A L    H O R I Z O N T A L    V E L O C I T Y                                         **
!***********************************************************************************************************************************

      subroutine initial_u_velocity
      
      USE GLOBAL; USE GEOMC
      use initialvelocity
      
      real :: xarea, wsurf
      integer :: k

      DO JW=1,NWB
        KT = KTWB(JW)
        
        DO JB=BS(JW),BE(JW)
          IF (SLOPE(JB) > 0.0 .and. .not. loop_branch(jb)) then
            IU = CUS(JB)
            ID = DS(JB)
            do i=iu,id
              wsurf=elws(i)              
              call xsectional_area(wsurf,xarea)
              uavg(i)=qssi(i)/xarea
              do k=kt,kb(i)
                u(k,i)=uavg(i)
              end do
            end do
          end if
        end do
      end do
                
      return
      end subroutine initial_u_velocity