
!***********************************************************************************************************************************
!**                                            S U B R O U T I N E   W I T H D R A W A L                                          **
!***********************************************************************************************************************************

SUBROUTINE WITHDRAWAL
  USE GLOBAL; USE GEOMC; USE TVDC; USE SELWC; USE LOGICC; USE MAIN, ONLY: DERIVED_CALC, CDN
  real :: hswt,hswb,elr,wsel,elstr,coef,ratio,ht,rhoft,dlrhot,hb,rhofb,dlrhob,vsum,dlrhomax,hwdt,hwdb,elwd,tempest,estrtest,QSUMJS
  REAL :: FRACV,QSUMWD
  integer :: k,js,kstr,ktop,kbot,jwd,kwd

RETURN

!***********************************************************************************************************************************
!**                                             D O W N S T R E A M   W I T H D R A W A L                                         **
!***********************************************************************************************************************************

ENTRY DOWNSTREAM_WITHDRAWAL (JS)

! Variable initialization

  HSWT = 0.0; HSWB = 0.0; VNORM = 0.0; QNEW = 0.0

! Water surface elevation

  ELR  = SINA(JB)*DLX(ID)*0.5
  WSEL = ELWS(ID)-ELR                   !EL(KT,ID)-Z(ID)*COSA(JB)

! Structure layer

  DO K=KT,KB(ID)
    IF (EL(K,ID)-ELR < ESTR(JS,JB)) EXIT
  END DO
  KSTR = MAX(K-1,KT)
  KSTR = MIN(KSTR,KB(ID))

! Initial withdrawal limits

  KTOP = MAX(KTSW(JS,JB),KT)
  IF (KSTR < KTOP) KTOP = KSTR
  KBOT = MIN(KBSW(JS,JB),KB(ID))
  IF (KBOT <= KT .AND. KBOT /= KB(ID)) KBOT = KT+1
  IF (KBOT > KB(ID)) KBOT = KB(ID)
  ELSTR = ESTR(JS,JB)
  IF (ESTR(JS,JB) <= EL(KB(ID)+1,ID+1)-ELR) THEN
    KSTR  = KB(ID)
    ELSTR = EL(KB(ID),ID)-ELR
  END IF
  IF (ESTR(JS,JB) > EL(KT,ID)-ELR) ELSTR = WSEL
  IF (KBSW(JS,JB) < KSTR) THEN
    KSTR  = KT
    ELSTR = WSEL
  END IF

! Boundary interference

  COEF = 1.0
  IF ((WSEL-EL(KBOT,ID)-ELR) /= 0.0) THEN
    RATIO = (ELSTR-(EL(KBOT,ID)-ELR))/(WSEL-(EL(KBOT,ID)-ELR))
    IF (RATIO < 0.1 .OR. RATIO > 0.9) COEF = 2.0
  END IF

! Withdrawal zone above structure

  DO K=KSTR-1,KTOP,-1

!** Density frequency

    HT    = (EL(K,ID)-ELR)-ELSTR
    RHOFT = MAX(SQRT((ABS(RHO(K,ID)-RHO(KSTR,ID)))/(HT*RHO(KSTR,ID)+NONZERO)*G),NONZERO)

!** Thickness

    IF (POINT_SINK(JS,JB)) THEN
      HSWT = (COEF*QSTR(JS,JB)/RHOFT)**0.333333
    ELSE
      HSWT = SQRT(2.0*COEF*QSTR(JS,JB)/(WSTR(JS,JB)*RHOFT))
    END IF
    IF (HT >= HSWT) THEN
      KTOP = K; EXIT
    END IF
  END DO

! Reference density

  IF ((ELSTR+HSWT) < WSEL) THEN
    DLRHOT = ABS(RHO(KSTR,ID)-RHO(KTOP,ID))
  ELSE IF (WSEL == ELSTR) THEN
    DLRHOT = NONZERO
  ELSE
    DLRHOT = ABS(RHO(KSTR,ID)-RHO(KT,ID))*HSWT/(WSEL-ELSTR)
  END IF
  DLRHOT = MAX(DLRHOT,NONZERO)

! Withdrawal zone below structure

  DO K=KSTR+1,KBOT

!** Density frequency

    HB    = ELSTR-(EL(K,ID)-ELR)
    RHOFB = MAX(SQRT((ABS(RHO(K,ID)-RHO(KSTR,ID)))/(HB*RHO(KSTR,ID)+NONZERO)*G),NONZERO)

!** Thickness

    IF (POINT_SINK(JS,JB)) THEN
      HSWB = (COEF*QSTR(JS,JB)/RHOFB)**0.333333
    ELSE
      HSWB = SQRT(2.0*COEF*QSTR(JS,JB)/(WSTR(JS,JB)*RHOFB))
    END IF
    IF (HB >= HSWB) THEN
      KBOT = K; EXIT
    END IF
  END DO

! Reference density

  IF ((ELSTR-HSWB) > EL(KBOT+1,ID)) THEN
    DLRHOB = ABS(RHO(KSTR,ID)-RHO(KBOT,ID))
  ELSE IF ((EL(KBOT+1,ID)-ELR) == ELSTR) THEN                                                                          !SR 03/24/13
    DLRHOB = NONZERO                                                                                                   !SR 03/24/13
  ELSE
    DLRHOB = ABS(RHO(KSTR,ID)-RHO(KBOT,ID))*HSWB/(ELSTR-(EL(KBOT+1,ID)-ELR))
  END IF
  DLRHOB = MAX(DLRHOB,NONZERO)

! Velocity profile

  VSUM     = 0.0
!  DLRHOMAX = MAX(DLRHOT,DLRHOB,1.0E-10)                      ! GH 1/31/08
  DO K=KTOP,KBOT
!    VNORM(K) = ABS(1.0-((RHO(K,ID)-RHO(KSTR,ID))/DLRHOMAX)**2)*BHR2(K,ID)
    IF(K.GT.KSTR)THEN
      DLRHOMAX = MAX(DLRHOB,1.0E-10)                          !GH 1/31/08
    ELSE
      DLRHOMAX = MAX(DLRHOT,1.0E-10)                          !GH 1/31/08
    ENDIF
    VNORM(K) = 1.0-((RHO(K,ID)-RHO(KSTR,ID))/DLRHOMAX)**2
    IF(VNORM(K).GT.1.0) VNORM(K)=1.0                         !GH 1/31/08
    IF(VNORM(K).LT.0.0) VNORM(K)=0.0                         !GH 1/31/08
    VNORM(K)=VNORM(K)*BHR2(K,ID)
    VSUM    =VSUM+VNORM(K)
  END DO

! Outflows
  qsumjs=0.0                                                  ! SW 7/30/09
  tavg(js,jb)=0.0                                                    ! cb 5/12/10
  if(constituents)cavg(js,jb,CN(1:NAC))=0.0
  if(derived_calc)cdavg(js,jb,CDN(1:nacd(jw),JW))=0.0
  DO K=KTOP,KBOT
    QNEW(K)    = (VNORM(K)/VSUM)*QSTR(JS,JB)
    QOUT(K,JB) =  QOUT(K,JB)+QNEW(K)
    tavg(js,jb)=tavg(js,jb)+qnew(k)*t2(k,id)                  ! SW 7/30/09
    if(constituents)cavg(js,jb,CN(1:NAC))=cavg(js,jb,CN(1:NAC))+qnew(k)*c2(k,id,cn(1:nac))
    if(derived_calc)cdavg(js,jb,CDN(1:nacd(JW),JW))=cdavg(js,jb,CDN(1:nacd(JW),JW))+qnew(k)*cd(k,id,CDN(1:nacd(JW),JW))
    qsumjs=qsumjs+qnew(k)
  END DO
if(qsumjs.gt.0.0)then
  tavg(js,jb)=tavg(js,jb)/qsumjs
  if(constituents)cavg(js,jb,CN(1:NAC))=cavg(js,jb,CN(1:NAC))/qsumjs
  if(derived_calc)cdavg(js,jb,CDN(1:nacd(JW),JW))=cdavg(js,jb,CDN(1:nacd(JW),JW))/qsumjs
else
  tavg(js,jb)=-99.0
  if(constituents)cavg(js,jb,CN(1:NAC))=-99.0
  if(derived_calc)cdavg(js,jb,CDN(1:nacd(JW),JW))=-99.0
end if

! Inactive layers and total outflow

  IF (JS == NST) THEN
    WHERE (QOUT(:,JB) == 0.0) U(:,ID) = 0.0
  END IF
RETURN

!***********************************************************************************************************************************
!**                                             D O W N S T R E A M   W I T H D R A W A L  ESTIMATE                               **
!***********************************************************************************************************************************

ENTRY DOWNSTREAM_WITHDRAWAL_estimate(JS,tempest,estrtest)

! Variable initialization

  HSWT = 0.0; HSWB = 0.0; VNORM = 0.0; QNEW = 0.0

! Water surface elevation

  ELR  = SINA(JB)*DLX(ID)*0.5
  WSEL = EL(KT,ID)-Z(ID)*COSA(JB)-ELR

! Structure layer

  DO K=KT,KB(ID)
    IF (EL(K,ID)-ELR < estrtest) EXIT
  END DO
  KSTR = MAX(K-1,KT)
  KSTR = MIN(KSTR,KB(ID))

! Initial withdrawal limits

  KTOP = MAX(KTSW(JS,JB),KT)
  IF (KSTR < KTOP) KTOP = KSTR
  KBOT = MIN(KBSW(JS,JB),KB(ID))
  IF (KBOT <= KT .AND. KBOT /= KB(ID)) KBOT = KT+1
  IF (KBOT > KB(ID)) KBOT = KB(ID)                                                                                     !SW 06/03/02
  ELSTR = estrtest
  IF (estrtest <= EL(KB(ID)+1,ID+1)-ELR) THEN                                                                       !SW 10/17/01
    KSTR  = KB(ID)
    ELSTR = EL(KB(ID),ID)-ELR                                                                                          !SW 10/17/01
  END IF
  IF (estrtest > EL(KT,ID)-ELR) ELSTR = WSEL
  IF (KBSW(JS,JB) < KSTR) THEN
    KSTR  = KT
    ELSTR = WSEL                                                                                                       !SW 10/05/00
  END IF

! Boundary interference

  COEF = 1.0
  IF ((WSEL-EL(KBOT,ID)-ELR) /= 0.0) THEN
    RATIO = (ELSTR-(EL(KBOT,ID)-ELR))/(WSEL-(EL(KBOT,ID)-ELR))                                                         !SW 10/17/01
    IF (RATIO < 0.1 .OR. RATIO > 0.9) COEF = 2.0
  END IF

! Withdrawal zone above structure

  DO K=KSTR-1,KTOP,-1

!** Density frequency

    HT    = (EL(K,ID)-ELR)-ELSTR
    RHOFT = MAX(SQRT((ABS(RHO(K,ID)-RHO(KSTR,ID)))/(HT*RHO(KSTR,ID)+NONZERO)*G),NONZERO)

!** Thickness

    IF (POINT_SINK(JS,JB)) THEN
      HSWT = (COEF*QSTR(JS,JB)/RHOFT)**0.333333
    ELSE
      HSWT = SQRT(2.0*COEF*QSTR(JS,JB)/(WSTR(JS,JB)*RHOFT))
    END IF
    IF (HT >= HSWT) THEN
      KTOP = K; EXIT
    END IF
  END DO

! Reference density

  IF ((ELSTR+HSWT) < WSEL) THEN
    DLRHOT = ABS(RHO(KSTR,ID)-RHO(KTOP,ID))
  ELSE IF (WSEL == ELSTR) THEN
    DLRHOT = NONZERO
  ELSE
    DLRHOT = ABS(RHO(KSTR,ID)-RHO(KT,ID))*HSWT/(WSEL-ELSTR)
  END IF
  DLRHOT = MAX(DLRHOT,NONZERO)

! Withdrawal zone below structure

  DO K=KSTR+1,KBOT

!** Density frequency

    HB    = ELSTR-(EL(K,ID)-ELR)                                                                                       !SW 10/17/01
    RHOFB = MAX(SQRT((ABS(RHO(K,ID)-RHO(KSTR,ID)))/(HB*RHO(KSTR,ID)+NONZERO)*G),NONZERO)

!** Thickness

    IF (POINT_SINK(JS,JB)) THEN
      HSWB = (COEF*QSTR(JS,JB)/RHOFB)**0.333333
    ELSE
      HSWB = SQRT(2.0*COEF*QSTR(JS,JB)/(WSTR(JS,JB)*RHOFB))
    END IF
    IF (HB >= HSWB) THEN
      KBOT = K; EXIT
    END IF
  END DO

! Reference density

  IF ((ELSTR-HSWB) > EL(KBOT+1,ID)) THEN
    DLRHOB = ABS(RHO(KSTR,ID)-RHO(KBOT,ID))
  ELSE IF ((EL(KBOT+1,ID)-ELR) == ELSTR) THEN                                                                          !SR 03/24/13
    DLRHOB = NONZERO                                                                                                   !SR 03/24/13
  ELSE
    DLRHOB = ABS(RHO(KSTR,ID)-RHO(KBOT,ID))*HSWB/(ELSTR-(EL(KBOT+1,ID)-ELR))                                           !SW 10/17/01
  END IF
  DLRHOB = MAX(DLRHOB,NONZERO)

! Velocity profile

  VSUM     = 0.0
!  DLRHOMAX = MAX(DLRHOT,DLRHOB,1.0E-10)
  DO K=KTOP,KBOT
    IF(K.GT.KSTR)THEN
      DLRHOMAX = MAX(DLRHOB,1.0E-10)                          !GH 1/31/08
    ELSE
      DLRHOMAX = MAX(DLRHOT,1.0E-10)                          !GH 1/31/08
    ENDIF
    VNORM(K) = 1.0-((RHO(K,ID)-RHO(KSTR,ID))/DLRHOMAX)**2
    IF(VNORM(K).GT.1.0) VNORM(K)=1.0                          !GH 1/31/08
    IF(VNORM(K).LT.0.0) VNORM(K)=0.0                          !GH 1/31/08
    VNORM(K)=VNORM(K)*BHR2(K,ID)
    VSUM    =VSUM+VNORM(K)
  END DO

! Outflows

  tempest=0.0
  DO K=KTOP,KBOT
    tempest=tempest+t2(k,id)*(VNORM(K)/VSUM)*QSTR(JS,JB)
  END DO

  if(qstr(js,jb).gt.0.0)tempest=tempest/qstr(js,jb)

RETURN

!***********************************************************************************************************************************
!**                                                L A T E R A L   W I T H D R A W A L                                            **
!***********************************************************************************************************************************

ENTRY LATERAL_WITHDRAWAL (JWD)

! Variable initialization

  VNORM = 0.0; QSW(:,JWD) = 0.0; HWDT = 0.0; HWDB = 0.0

! Structure layer

  K = KT
  DO K=KT,KB(I)
    IF (EL(K,I) < EWD(JWD)) EXIT
  END DO
  KWD = MAX(K-1,KT)
  KWD = MIN(KWD,KB(I))

! Initial withdrawal limits

  KTOP = MAX(KTWD(JWD),KT)
  IF (KWD < KTOP) KTOP = KWD
  KBOT = MIN(KBWD(JWD),KB(I))
  IF (KBOT <= KT .AND. KB(I) /= KBOT) KBOT = KT+1
  IF (KBOT > KB(I)) KBOT = KB(I)
  ELWD = EWD(JWD)
  IF (EWD(JWD) <= EL(KB(I)+1,I)) THEN
    KWD  = KB(I)
    ELWD = EL(KB(I),I)
  END IF
  IF (EWD(JWD) > EL(KT,I)) ELWD = EL(KT,I)
  IF (KBWD(JWD) < KWD) THEN
    KWD  = KT
    ELWD = EL(KT,I)
  END IF

! Boundary interference

  COEF = 1.0
  IF (KT /= KBOT) THEN
    RATIO = (ELWD-EL(KBOT,I))/(EL(KT,I)-EL(KBOT,I))
    IF (RATIO < 0.1 .OR. RATIO > 0.9) COEF = 2.0
  END IF

! Withdrawal zone above structure

  DO K=KWD-1,KTOP,-1

!** Density frequency

    HT    = EL(K,I)-ELWD
    RHOFT = MAX(SQRT((ABS(RHO(K,I)-RHO(KWD,I)))/(HT*RHO(KWD,I)+NONZERO)*G),NONZERO)

!** Thickness

    HWDT = (COEF*QWD(JWD)/RHOFT)**0.333333
    IF (HT >= HWDT) THEN
      KTOP = K; EXIT
    END IF
  END DO

! Reference density

  IF ((ELWD+HWDT) < EL(KT,I)) THEN
    DLRHOT = ABS(RHO(KWD,I)-RHO(KTOP,I))
  ELSE IF (EL(KT,I) == ELWD) THEN
    DLRHOT = NONZERO
  ELSE
    DLRHOT = ABS(RHO(KWD,I)-RHO(KT,I))*HWDT/(EL(KT,I)-ELWD)
  END IF
  DLRHOT = MAX(DLRHOT,NONZERO)

! Withdrawal zone below structure

  DO K=KWD+1,KBOT

!** Density frequency

    HB    = ELWD-EL(K,I)
    RHOFB = MAX(SQRT((ABS(RHO(K,I)-RHO(KWD,I)))/(HB*RHO(KWD,I)+NONZERO)*G),NONZERO)

!** Thickness

    HWDB = (COEF*QWD(JWD)/RHOFB)**0.333333
    IF (HB >= HWDB) THEN
      KBOT = K; EXIT
    END IF
  END DO

! Reference density

  IF ((ELWD-HWDB) > EL(KBOT+1,I)) THEN
    DLRHOB = ABS(RHO(KWD,I)-RHO(KBOT,I))
  ELSE IF (EL(KBOT+1,I) == ELWD) THEN                                                                                  !SR 03/24/13
    DLRHOB = NONZERO                                                                                                   !SR 03/24/13
  ELSE
    DLRHOB = ABS(RHO(KWD,I)-RHO(KBOT,I))*HWDB/(ELWD-EL(KBOT+1,I))
  END IF
  DLRHOB = MAX(DLRHOB,NONZERO)

! Velocity profile

  VSUM     = 0.0
!  DLRHOMAX = MAX(DLRHOT,DLRHOB,1.0E-10)                                                                             ! SW 1/24/05
  DO K=KTOP,KBOT
!    VNORM(K) = ABS(1.0-((RHO(K,I)-RHO(KWD,I))/DLRHOMAX)**2)*BHR2(K,I)
    IF(K.GT.KWD)THEN
      DLRHOMAX = MAX(DLRHOB,1.0E-10)                          !GH 1/31/08
    ELSE
      DLRHOMAX = MAX(DLRHOT,1.0E-10)                          !GH 1/31/08
    ENDIF
    VNORM(K) = 1.0-((RHO(K,I)-RHO(KWD,I))/DLRHOMAX)**2
    IF(VNORM(K).GT.1.0) VNORM(K)=1.0                         !GH 1/31/08
    IF(VNORM(K).LT.0.0) VNORM(K)=0.0                         !GH 1/31/08
    VNORM(K)=VNORM(K)*BHR2(K,I)
    VSUM    =VSUM+VNORM(K)
  END DO

! Outflows
  qsumwd=0.0                                                  ! SW 7/30/09
  tavgw(jwd)=0.0
  if(constituents)cavgw(jwd,CN(1:NAC))=0.0
  if(derived_calc)cdavgw(jwd,CDN(1:nacd(JW),JW))=0.0

  DO K=KTOP,KBOT
    fracv=(VNORM(K)/VSUM)
    QSW(K,JWD) = QSW(K,JWD)+fracv*QWD(JWD)
    tavgw(jwd)=tavgw(jwd)+fracv*QWD(JWD)*t2(k,i)                  ! SW 7/30/09
    if(constituents)cavgw(jwd,CN(1:NAC))=cavgw(jwd,CN(1:NAC))+fracv*qwd(jwd)*c2(k,i,cn(1:nac))
    if(derived_calc)cdavgw(jwd,CDN(1:nacd(JW),JW))=cdavgw(jwd,CDN(1:nacd(JW),JW))+fracv*qwd(jwd)*cd(k,i,CDN(1:nacd(JW),JW))
    qsumwd=qsumwd+fracv*QWD(JWD)
  END DO
  if(qsumwd.gt.0.0)then
    tavgw(jwd)=tavgw(jwd)/qsumwd               ! SW 7/30/09
    if(constituents)cavgw(jwd,CN(1:NAC))=cavgw(jwd,CN(1:NAC))/qsumwd
    if(derived_calc)cdavgw(jwd,CDN(1:nacd(JW),JW))=cdavgw(jwd,CDN(1:nacd(JW),JW))/qsumwd
    else
    tavgw(jwd)=-99.0
    if(constituents)cavgw(jwd,CN(1:NAC))=-99.0
    if(derived_calc)cdavgw(jwd,CDN(1:nacd(JW),JW))=-99.0
  endif
  KTW(JWD) = KTOP
  KBW(JWD) = KBOT
return

!***********************************************************************************************************************************
!**                                                L A T E R A L   W I T H D R A W A L ESTIMATE                                   **
!***********************************************************************************************************************************

ENTRY LATERAL_WITHDRAWAL_ESTIMATE (JWD,tempest,estrtest)

! Variable initialization

  VNORM = 0.0; QSW(:,JWD) = 0.0; HWDT = 0.0; HWDB = 0.0

! Structure layer

  K = KT
  DO K=KT,KB(I)
    IF (EL(K,I) < estrtest) EXIT
  END DO
  KWD = MAX(K-1,KT)
  KWD = MIN(KWD,KB(I))

! Initial withdrawal limits

  KTOP = MAX(KTWD(JWD),KT)
  IF (KWD < KTOP) KTOP = KWD
  KBOT = MIN(KBWD(JWD),KB(I))
  IF (KBOT <= KT .AND. KB(I) /= KBOT) KBOT = KT+1
  IF (KBOT > KB(I)) KBOT = KB(I)
  ELWD = estrtest
  IF (estrtest <= EL(KB(I)+1,I)) THEN
    KWD  = KB(I)
    ELWD = EL(KB(I),I)
  END IF
  IF (estrtest > EL(KT,I)) ELWD = EL(KT,I)
  IF (KBWD(JWD) < KWD) THEN
    KWD  = KT
    ELWD = EL(KT,I)
  END IF

! Boundary interference

  COEF = 1.0
  IF (KT /= KBOT) THEN
    RATIO = (ELWD-EL(KBOT,I))/(EL(KT,I)-EL(KBOT,I))
    IF (RATIO < 0.1 .OR. RATIO > 0.9) COEF = 2.0
  END IF

! Withdrawal zone above structure

  DO K=KWD-1,KTOP,-1

!** Density frequency

    HT    = EL(K,I)-ELWD
    RHOFT = MAX(SQRT((ABS(RHO(K,I)-RHO(KWD,I)))/(HT*RHO(KWD,I)+NONZERO)*G),NONZERO)

!** Thickness

    HWDT = (COEF*QWD(JWD)/RHOFT)**0.333333
    IF (HT >= HWDT) THEN
      KTOP = K; EXIT
    END IF
  END DO

! Reference density

  IF ((ELWD+HWDT) < EL(KT,I)) THEN
    DLRHOT = ABS(RHO(KWD,I)-RHO(KTOP,I))
  ELSE IF (EL(KT,I) == ELWD) THEN
    DLRHOT = NONZERO
  ELSE
    DLRHOT = ABS(RHO(KWD,I)-RHO(KT,I))*HWDT/(EL(KT,I)-ELWD)
  END IF
  DLRHOT = MAX(DLRHOT,NONZERO)

! Withdrawal zone below structure

  DO K=KWD+1,KBOT

!** Density frequency

    HB    = ELWD-EL(K,I)
    RHOFB = MAX(SQRT((ABS(RHO(K,I)-RHO(KWD,I)))/(HB*RHO(KWD,I)+NONZERO)*G),NONZERO)

!** Thickness

    HWDB = (COEF*QWD(JWD)/RHOFB)**0.333333
    IF (HB >= HWDB) THEN
      KBOT = K; EXIT
    END IF
  END DO

! Reference density

  IF ((ELWD-HWDB) > EL(KBOT+1,I)) THEN
    DLRHOB = ABS(RHO(KWD,I)-RHO(KBOT,I))
  ELSE IF (EL(KBOT+1,I) == ELWD) THEN                                                                                  !SR 03/24/13
    DLRHOB = NONZERO                                                                                                   !SR 03/24/13
  ELSE
    DLRHOB = ABS(RHO(KWD,I)-RHO(KBOT,I))*HWDB/(ELWD-EL(KBOT+1,I))
  END IF
  DLRHOB = MAX(DLRHOB,NONZERO)

! Velocity profile

  VSUM     = 0.0
!  DLRHOMAX = MAX(DLRHOT,DLRHOB,1.0E-10)                                                                             ! SW 1/24/05
  DO K=KTOP,KBOT
!    VNORM(K) = ABS(1.0-((RHO(K,I)-RHO(KWD,I))/DLRHOMAX)**2)*BHR2(K,I)
    IF(K.GT.KWD)THEN
      DLRHOMAX = MAX(DLRHOB,1.0E-10)                          !GH 1/31/08
    ELSE
      DLRHOMAX = MAX(DLRHOT,1.0E-10)                          !GH 1/31/08
    ENDIF
    VNORM(K) = 1.0-((RHO(K,I)-RHO(KWD,I))/DLRHOMAX)**2
    IF(VNORM(K).GT.1.0) VNORM(K)=1.0                          !GH 1/31/08
    IF(VNORM(K).LT.0.0) VNORM(K)=0.0                          !GH 1/31/08
    VNORM(K)=VNORM(K)*BHR2(K,I)
    VSUM    =VSUM+VNORM(K)
  END DO

! Outflows

  tempest=0.0                                                                                                          !SR 03/24/13
  DO K=KTOP,KBOT
    tempest=tempest+t2(k,i)*(VNORM(K)/VSUM)*QWD(JWD)
  END DO
  if(qwd(jwd).gt.0.0)tempest=tempest/qwd(jwd)
  KTW(JWD) = KTOP
  KBW(JWD) = KBOT
return

END SUBROUTINE WITHDRAWAL


!***********************************************************************************************************************************
!**                                                S E L E C T I V E   I N I T                                                    **
!***********************************************************************************************************************************

Module Selective1
  INTEGER                                      :: numtempc, numtsplt, tempn, ng1, ng2
  INTEGER,       ALLOCATABLE, DIMENSION(:)     :: tcnelev, tcjb, tcjs, tciseg, kstrsplt, ncountcw, seld
  INTEGER,       ALLOCATABLE, DIMENSION(:)     :: tspltjb, tsseld, nouts, nout0, nout1, nout2
  INTEGER,       ALLOCATABLE, DIMENSION(:,:)   :: jstsplt, ncountc, tsprior
  REAL                                         :: nxtstr, nxttcd, nxtsplit, tcdfreq, tfrqtmp, tspltfreq
  REAL                                         :: sum_minfrac1, sum_minfrac2, qfrac1, qfrac2, tsconv
  REAL,          ALLOCATABLE, DIMENSION(:)     :: tctemp, tctend, tctsrt, tcklay, tspltt, volm, qwdfrac, tstend, tstsrt, nxsel,temp2
  REAL,          ALLOCATABLE, DIMENSION(:)     :: nxtssel, tstemp2, ewdsav, minfrac1, minfrac2, maxfrac1, maxfrac2, splt2t, splt2e
  REAL,          ALLOCATABLE, DIMENSION(:,:)   :: tcelev, tempcrit, qstrfrac, volmc
  REAL,          ALLOCATABLE, DIMENSION(:,:)   :: tsdepth, tsminfrac, tsminhead, tsmaxhead, tsmaxflow, estrsav
  CHARACTER(8)                                 :: tempc, tspltc
  CHARACTER(8),  ALLOCATABLE, DIMENSION(:)     :: tcelevcon, tcyearly, tcntr, dynsel
  CHARACTER(8),  ALLOCATABLE, DIMENSION(:)     :: tspltcntr, tsyearly, elcontspl, tsdynsel
  CHARACTER(5),  ALLOCATABLE, DIMENSION(:,:)   :: tstype
  LOGICAL,       ALLOCATABLE, DIMENSION(:)     :: wd_active, share_flow
  LOGICAL,       ALLOCATABLE, DIMENSION(:,:)   :: no_flow, str_active
End Module Selective1


Subroutine SelectiveInit

  Use Selective1; USE MAIN
  USE GLOBAL;     USE NAMESC; USE GEOMC;  USE LOGICC; USE PREC;  USE SURFHE;  USE KINETIC; USE SHADEC; USE EDDY
  USE STRUCTURES; USE TRANS;  USE TVDC;   USE SELWC;  USE GDAYC; USE SCREENC; USE TDGAS;   USE RSTART

  integer      :: ifile, nj
  character(8) :: tsshare

  ifile=1949
  tavg=0.0
  tavgw=0.0
  do jb=1,nbr
    if(nstr(jb) > 0)then
      ifile=ifile+1
      write (segnum,'(i0)') jb
      segnum = adjustl(segnum)
      l      = len_trim(segnum)

      IF(RESTART_IN)THEN
        OPEN  (ifile,file='str_br'//segnum(1:l)//'.opt',POSITION='APPEND')
        JDAY1=0.0
        REWIND (IFILE)
        READ   (IFILE,'(/)',END=13)
        DO WHILE (JDAY1 < JDAY)
          READ (IFILE,'(F10.0)',END=13) JDAY1
        END DO
        BACKSPACE (IFILE)
13      JDAY1=0.0
      ELSE
        open  (ifile,file='str_br'//segnum(1:l)//'.opt',status='unknown')
        write (ifile,*)'Branch:',jb,' # of structures:',nstr(jb),' outlet temperatures'
        write (ifile,'("      JDAY",<nstr(jb)>(6x,"T(C)"),<nstr(jb)>(3x,"Q(m3/s)"),<nstr(jb)>(4x,"ELEVCL"))')
      ENDIF
    endif
  end do

  if(nwd > 0)then
    ifile=ifile+1
    IF(RESTART_IN)THEN
      OPEN  (ifile,file='wd_out.opt',POSITION='APPEND')
      JDAY1=0.0
      REWIND (IFILE)
      READ   (IFILE,'(/)',END=14)
      DO WHILE (JDAY1 < JDAY)
        READ (IFILE,'(F10.0)',END=14) JDAY1
      END DO
      BACKSPACE (IFILE)
14    JDAY1=0.0
    ELSE
      open  (ifile,file='wd_out.opt',status='unknown')
      write (ifile,*)'Withdrawals: # of withdrawals:',nwd,' outlet temperatures'
      write (ifile,'("      JDAY",<nwd>(6x,"T(C)"),<nwd>(3x,"Q(m3/s)"),<nwd>(4x,"ELEVCL"))')
    ENDIF
  end if

  open (1010,file='w2_selective.npt',status='old')
  read (1010,'(///8x,f8.0)') tfrqtmp
  read (1010,'(//8x,a8,i8,f8.0)') tempc, numtempc, tcdfreq
  nxtstr   = tmstrt
  nxttcd   = tmstrt
  nxtsplit = tmstrt

  allocate (tcnelev(numtempc),tcjb(numtempc),tcjs(numtempc),tcelev(numtempc,11),tctemp(numtempc),tctend(numtempc),tctsrt(numtempc))
  allocate (ncountc(nst,nbr),tciseg(numtempc),tcklay(numtempc),tcelevcon(numtempc))
  Allocate (tcyearly(numtempc), tcntr(numtempc))
  allocate (volm(nwb),ncountcw(nwd),qwdfrac(nwd),qstrfrac(nst,nbr),DYNSEL(numtempc),SELD(numtempc),NXSEL(numtempc),TEMP2(numtempc))

  ncountc=0
  read (1010,'(/)')
  do j=1,numtempc
    read(1010,'(8x,a8,i8,i8,a8,f8.0,f8.0,f8.0,i8,10(f8.0))')tcntr(j),tcjb(j),tcjs(j),tcyearly(j),tctsrt(j),tctend(j),tctemp(j),tcnelev(j),(tcelev(j,n),n=1,tcnelev(j))
    if(tcntr(j)=='      ST')then
      tcelev(j,tcnelev(j)+1)=ESTR(tcjs(j),tcjb(j))   ! always put the original elevation as the last elevation
    else
      tcelev(j,tcnelev(j)+1)=EWD(tcjs(j))   ! always put the original elevation as the last elevation
    endif
  end do
  read (1010,'(/)')
  do j=1,numtempc
    read (1010,'(8x,i8,f8.0,A8)') tciseg(j), tcklay(j), DYNSEL(J)
  end do
  read (1010,'(/)')
  do j=1,numtempc
    read (1010,'(8x,a8)') tcelevcon(j)
  end do
  read (1010,'(//8x,a8,i8,2f8.0)') tspltc, numtsplt, tspltfreq, tsconv

  allocate (tsyearly(numtsplt), tstsrt(numtsplt), tstend(numtsplt), tspltjb(numtsplt), tspltt(numtsplt), nouts(numtsplt))
  allocate (jstsplt(numtsplt,10), kstrsplt(10), tspltcntr(numtsplt), elcontspl(numtsplt))
  allocate (tsdepth(numtsplt,10), tstype(numtsplt,10), tsminfrac(numtsplt,10), tsprior(numtsplt,10))
  allocate (tsminhead(numtsplt,10), tsmaxhead(numtsplt,10), tsmaxflow(numtsplt,10), no_flow(numtsplt,10), share_flow(numtsplt))
  allocate (tsdynsel(numtsplt), tsseld(numtsplt), nxtssel(numtsplt), tstemp2(numtsplt))
  allocate (nout0(10), nout1(10), nout2(10), minfrac1(10), maxfrac1(10), minfrac2(10), maxfrac2(10), splt2t(10), splt2e(10))
  allocate (ewdsav(nwd), wd_active(nwd), estrsav(nst,nbr), str_active(nst,nbr))

  read (1010,'(/)')
  do j=1,numtsplt
    read (1010,'(8x,a8,i8,a8,3f8.0,2a8,i8,a8)') tspltcntr(j), tspltjb(j), tsyearly(j), tstsrt(j), tstend(j),                       &
                                                tspltt(j), tsdynsel(j), elcontspl(j), nouts(j), tsshare
    if (tspltc == '      ON') then
      if (nouts(j) < 2) then
        write (w2err, '(A,I0)') 'ERROR-- Less than two outlets specified for blending group ',j
        ERROR_OPEN = .TRUE.     ! will trigger the program to end when this subroutine is completed
        return
      else if (nouts(j) > 10) then
        write (w2err, '(A,I0)') 'ERROR-- More than ten outlets specified for blending group ',j
        ERROR_OPEN = .TRUE.
        return
      end if
    end if
    share_flow(j) = tsshare == '      ON'
  end do
  read (1010,'(/)')
  do j=1,numtsplt
    read (1010,'(8x,10i8)') (jstsplt(j,n),n=1,nouts(j))
  end do
  read (1010,'(/)')
  do j=1,numtsplt
    read (1010,'(8x,10f8.0)') (tsdepth(j,n),n=1,nouts(j))
  end do
  read (1010,'(/)')
  do j=1,numtsplt
    read (1010,'(8x,10f8.0)') (tsminfrac(j,n),n=1,nouts(j))
  end do
  read (1010,'(/)')
  do j=1,numtsplt
    read (1010,'(8x,10i8)') (tsprior(j,n),n=1,nouts(j))
  end do
  read (1010,'(/)')
  do j=1,numtsplt
    read (1010,'(8x,10f8.0)') (tsminhead(j,n),n=1,nouts(j))
  end do
  read (1010,'(/)')
  do j=1,numtsplt
    read (1010,'(8x,10f8.0)') (tsmaxhead(j,n),n=1,nouts(j))
  end do
  read (1010,'(/)')
  do j=1,numtsplt
    read (1010,'(8x,10f8.0)') (tsmaxflow(j,n),n=1,nouts(j))
  end do

  estrsav = estr    ! Save the original structure elevations
  ewdsav  = ewd     ! Save the original withdrawal elevations
  do j=1,numtsplt
    do n=1,nouts(j)
      tstype(j,n) = "FIXED"
      if (tsdepth(j,n)   > 0.0) tstype(j,n) = "FLOAT"
      if (tsminfrac(j,n) > 1.0) tsminfrac(j,n) = 1.0    ! remove unrealistic input value
      if (tsminhead(j,n) < 0.0) tsminhead(j,n) = 0.0    ! remove unrealistic input value
      if (tsmaxhead(j,n) < 0.0) tsmaxhead(j,n) = 0.0    ! remove unrealistic input value
      if (tsmaxflow(j,n) < 0.0) tsmaxflow(j,n) = 0.0    ! remove unrealistic input value
    end do
  end do
  if (tsconv <= 0.0) tsconv = 0.005   ! constrain the convergence criterion to be > 0.0 and <= 0.1
  if (tsconv >  0.1) tsconv = 0.1

  read (1010,'(//8x,i8)') tempn
  allocate (tempcrit(nwb,tempn),volmc(nwb,tempn))
  read (1010,'(/)')
  do j=1,tempn
    read (1010,'(8x,10f8.0)') (tempcrit(jw,j), jw=1,nwb)   ! Note max of 10 waterbodies
  end do
  close (1010)

  do jw=1,nwb
    ifile=ifile+1
    write (segnum,'(i0)') jw
    segnum = adjustl(segnum)
    l      = len_trim(segnum)
    IF(RESTART_IN)THEN
      OPEN  (ifile,file='Volume_wb'//segnum(1:l)//'.opt',POSITION='APPEND')
      JDAY1=0.0
      REWIND (IFILE)
      READ   (IFILE,'(/)',END=15)
      DO WHILE (JDAY1 < JDAY)
        READ (IFILE,'(F10.0)',END=15) JDAY1
      END DO
      BACKSPACE (IFILE)
15    JDAY1=0.0
    ELSE
      open (ifile,file='Volume_wb'//segnum(1:l)//'.opt',status='unknown')
      write(ifile,4315)
    END IF
  end do

4315  format("jday    Volume    ",<tempn>("Volcrit      "))

  if (tempc == '      ON') then
    do j=1,numtempc
      if (tcyearly(j) == '     OFF') then
        daytest=jday
      else
        daytest=real(jdayg)+jday-int(jday)
      end if
      if (daytest >= tctsrt(j) .and. daytest < tctend(j)) then

      ! initializing structure elevation
        if (tcntr(j) == '      ST') then
          jb = tcjb(j)                                         ! set branch index
          js = tcjs(j)                                         ! set structure index
          do nj=1,tcnelev(j)                                   ! making sure that structure is below water surface
            if (tcelev(j,nj) < elws(ds(jb))) then
              ncountc(js,jb)=nj
            ! ESTR(js,jb)=tcelev(j,ncountc(js,jb))    ! don't alter the elevation at this point.  Set it later.
              exit
            end if
          end do

      ! initializing withdrawal elevation
        else if (tcntr(j) == '      WD') then
          jwd = tcjs(j)                                        ! set withdrawal index
          do nj=1,tcnelev(j)                                   ! making sure that structure is below water surface
            if (tcelev(j,nj) < elws(iwd(jwd))) then
              ncountcw(jwd)=nj
            ! EWD(jwd)=tcelev(j,ncountcw(jwd))        ! don't alter the elevation at this point.  Set it later.
              exit
            end if
          end do
        end if
      end if

    ! Open dynamic selective withdrawal files
      if (DYNSEL(J) == '      ON') then
        WRITE (SEGNUM,'(I0)') J
        SEGNUM  = ADJUSTL(SEGNUM)
        L       = LEN_TRIM(SEGNUM)
        SELD(J) = 1009+J
        OPEN (SELD(J),FILE='dynselective'//SEGNUM(1:L)//'.npt',STATUS='OLD')
        READ (SELD(J),'(///1000F8.0)') NXSEL(J),TEMP2(J)
        tctemp(J)=TEMP2(J)
        READ (SELD(J),'(1000F8.0)') NXSEL(J),TEMP2(J)
      end if
    end do
  end if

! Open dynamic temperature target files for blending outlets
  if (tspltc == '      ON') then
    do j=1,numtsplt
      if (tsdynsel(j) == '      ON') then
        write (segnum,'(i0)') j
        segnum    = adjustl(segnum)
        L         = len_trim(segnum)
        tsseld(j) = 1009+numtempc+j
        open (tsseld(j),file='dynsplit_selective'//segnum(1:L)//'.npt',status='old')
        read (tsseld(j),'(///2F8.0)') nxtssel(j), tstemp2(j)
        tspltt(j) = tstemp2(j)
        read (tsseld(j),'(2F8.0)') nxtssel(j), tstemp2(j)
      end if
    end do
  end if

! Test to see if the user specified inconsistent inputs. If so, stop with an error message.
  if (tspltc == '      ON') then
    do j=1,numtsplt
      do n=1,nouts(j)-1
        do nj=n+1,nouts(j)
          if (jstsplt(j,n) == jstsplt(j,nj)) then
            write (w2err, '(A,I0)') 'ERROR-- Duplicate split outlet numbers in group ', j
            ERROR_OPEN = .TRUE.     ! will trigger the program to end when this subroutine is completed
          end if
        end do
      end do
    end do
    do j=1,numtsplt-1
      do jj=j+1,numtsplt
        if ((tstsrt(jj) >= tstsrt(j) .and. tstsrt(jj) <  tstend(j)) .or.                                                           &
            (tstend(jj) >  tstsrt(j) .and. tstend(jj) <= tstend(j))) then
          if (tspltcntr(j) .eq. tspltcntr(jj) .and. (tspltcntr(j) .eq. '      WD' .or. tspltjb(j) == tspltjb(jj))) then
            do n=1,nouts(j)
              do nj=1,nouts(jj)
                if (jstsplt(j,n) == jstsplt(jj,nj)) then
                  write (w2err, '(A,I0,A)') 'ERROR-- Split outlet number ', jstsplt(j,n), ' used in more than one group at a time.'
                  ERROR_OPEN = .TRUE.       ! will trigger the program to end when this subroutine is completed
                end if
              end do
            end do
          end if
        end if
      end do
    end do
    do j=1,numtsplt
      do n=1,nouts(j)
        if (tsprior(j,n) < -1) then
          write (w2err, '(A,I0,A,I0,A)') 'ERROR-- Priority input for outlet ', jstsplt(j,n), ' in group ', j, ' is less than -1.'
          ERROR_OPEN = .TRUE.            ! will trigger the program to end when this subroutine is completed
        end if
        if (tsminhead(j,n) > 0.0 .and. tsmaxhead(j,n) > 0.0 .and. tsminhead(j,n) > tsmaxhead(j,n)) then
          write (wrn, '(A,I0,A,I0,A)') 'WARNING-- Minimum and maximum head constraints for outlet ', jstsplt(j,n), ' in group ',   &
                                        j, ' are such that the outlet cannot ever be used.'
          WARNING_OPEN = .TRUE.
        end if
        if (tsdepth(j,n) > 0.0 .and. tsminhead(j,n) > 0.0 .and. tsdepth(j,n) < tsminhead(j,n)) then
          write (wrn, '(A,I0,A,I0,A)') 'WARNING-- Depth of floating outlet ', jstsplt(j,n), ' in group ', j,                       &
              ' is shallower than the minimum head constraint.  To honor the head constraint, no flow is possible for that outlet.'
          WARNING_OPEN = .TRUE.
        end if
      end do
    end do

    if (tempc == '      ON') then
      do j=1,numtsplt
        do jj=1,numtempc
          if ((tctsrt(jj) >= tstsrt(j) .and. tctsrt(jj) <  tstend(j)) .or.                                                         &
              (tctend(jj) >  tstsrt(j) .and. tctend(jj) <= tstend(j))) then
            if (tspltcntr(j) .eq. tcntr(jj) .and. (tspltcntr(j) .eq. '      WD' .or. tspltjb(j) == tcjb(jj))) then
              do n=1,nouts(j)
                if (jstsplt(j,n) == tcjs(jj)) then
                  write (w2err, '(A,I0,A)') 'ERROR-- Outlet number ',tcjs(jj),' used in tower and blending group at same time.'
                  ERROR_OPEN = .TRUE.       ! will trigger the program to end when this subroutine is completed
                end if
              end do
            end if
          end if
        end do
      end do
    end if
  end if

  if (tempc == '      ON') then
    do j=1,numtempc-1
      do jj=j+1,numtempc
        if ((tctsrt(jj) >= tctsrt(j) .and. tctsrt(jj) <  tctend(j)) .or.                                                           &
            (tctend(jj) >  tctsrt(j) .and. tctend(jj) <= tctend(j))) then
          if (tcntr(j) .eq. tcntr(jj) .and. (tcntr(j) .eq. '      WD' .or. tcjb(j) == tcjb(jj))) then
            if (tcjs(j) == tcjs(jj)) then
              write (w2err, '(A,I0,A)') 'ERROR-- Tower outlet number ', tcjs(j), ' used more than once for overlapping dates.'
              ERROR_OPEN = .TRUE.       ! will trigger the program to end when this subroutine is completed
            end if
          end if
        end if
      end do
    end do
  end if

return

End subroutine SelectiveInit


!***********************************************************************************************************************************
!**                                                   S E L E C T I V E                                                           **
!***********************************************************************************************************************************

Subroutine Selective
  Use Selective1; USE MAIN
  USE GLOBAL;     USE NAMESC; USE GEOMC;  USE LOGICC; USE PREC;  USE SURFHE;  USE KINETIC; USE SHADEC; USE EDDY
  USE STRUCTURES; USE TRANS;  USE TVDC;   USE SELWC;  USE GDAYC; USE SCREENC; USE TDGAS;   USE RSTART

  integer :: jj, jst, n, nj, num_noflow, ng0, prior1, prior2, ng1max, ng1min, num_left
  integer :: j2hi, j2lo, j2max, j2min, j2pref
  real    :: qall, elr, wsel, q_notblended, sum_minfrac0, sum_maxfrac1, sum_maxfrac2, sumfrac
  real    :: maxelev, minelev, blendfrac, excess_frac, addfrac, maxtemp, mintemp
  real    :: lastfrac, lastfrac2, ttarg, sumtemp, etemp, etemp1, etemp2, sumelev, elev1, elev2

! qstr = qstrsav   ! xxx not sure how to do this yet -- need to reset QSTR when control periods expire
! qwd  = qwdsav    ! xxx not sure how to do this yet -- need to reset QWD when control periods expire

  str_active = .FALSE.
  wd_active  = .FALSE.
  j2pref     = 1

! Update some date variables and determine which outlets are being actively blended or adjusted.
  if (tspltc == '      ON') then
    do j=1,numtsplt
      if (tsyearly(j) == '     OFF') then
        daytest = jday
      else
        daytest = real(jdayg) + jday - int(jday)
      end if
      if (nxtsplit > tstsrt(j) .and. daytest <= tstsrt(j)) then
        nxtsplit = tstsrt(j)
      end if
      if (daytest >= tstsrt(j) .and. daytest < tstend(j)) then
        do jj=1,nouts(j)
          if (tspltcntr(j) == '      ST') then
            str_active(jstsplt(j,jj),tspltjb(j)) = .TRUE.
          else if (tspltcntr(j) == '      WD') then
            wd_active(jstsplt(j,jj)) = .TRUE.
          end if
        end do
      end if
    end do
  end if
  if (tempc == '      ON') then
    do j=1,numtempc
      if (tcyearly(j) == '     OFF') then
        daytest = jday
      else
        daytest = real(jdayg) + jday - int(jday)
      end if
      if (daytest >= tctsrt(j) .and. daytest < tctend(j)) then
        if (tcntr(j) == '      ST') then
          str_active(tcjs(j),tcjb(j)) = .TRUE.
        else if (tcntr(j) == '      WD') then
          wd_active(tcjs(j)) = .TRUE.
        end if
      end if
    end do
  end if

! Reset elevations of outlets back to original values outside of control periods
  do jst=1,nst
    do jb=1,nbr
      if (.not. str_active(jst,jb)) estr(jst,jb) = estrsav(jst,jb)
    end do
  end do
  do jwd=1,nwd
    if (.not. wd_active(jwd)) ewd(jwd) = ewdsav(jwd)
  end do


! Check to see if it's time to update temperature targets and flow fractions for blended groups.
  if (tspltc=='      ON' .and. jday .ge. nxtsplit) then

  ! Update the temperature targets
    do j=1,numtsplt
      if (tsdynsel(j) == '      ON') then
        do while (jday >= nxtssel(j))
          tspltt(j) = tstemp2(j)
          read (tsseld(j),'(2F8.0)') nxtssel(j), tstemp2(j)
        end do
      end if
    end do

    do j=1,numtsplt
      qall    = 0.0                                                          ! sum up all the flows
      sumfrac = 0.0                                                          ! sum of flow fraction multipliers
      do jj=1,nouts(j)
        if (tspltcntr(j) == '      ST') then
          qall    = qall    + qstr(jstsplt(j,jj),tspltjb(j))
          sumfrac = sumfrac + qstrfrac(jstsplt(j,jj),tspltjb(j))
        else if (tspltcntr(j) == '      WD') then
          qall    = qall    + qwd(jstsplt(j,jj))
          sumfrac = sumfrac + qwdfrac(jstsplt(j,jj))
        end if
      end do
      if (tsyearly(j) == '     OFF') then
        daytest = jday
      else
        daytest = real(jdayg) + jday - int(jday)
      end if

    ! Do blending calculations if date is in correct window or the window was just entered.
    ! If flows are zero and flow fractions are already initialized, then leave everything alone.
      if (daytest >= tstsrt(j) .and. daytest < tstend(j) .and.                                                                     &
          (qall > 0.0 .or. sumfrac < 0.001 .or. daytest < tstsrt(j) + tspltfreq)) then

      ! Do structures first
        if (tspltcntr(j) == '      ST') then
          jb = tspltjb(j)                                                    ! set branch index
          do jw=1,nwb                                                        ! set waterbody index
            if (jb >= bs(jw) .and. jb <= be(jw)) exit
          end do
          no_flow(j,:) = .FALSE.
          num_noflow   = 0
          do jj=1,nouts(j)
            jst          = jstsplt(j,jj)
            elr          = sina(jb) * dlx(ds(jb)) * 0.5
            wsel         = elws(ds(jb)) - elr                                ! compute water-surface elevation
            estr(jst,jb) = estrsav(jst,jb)                                   ! reset outlet elevation to original
            if (tstype(j,jj) .eq. "FLOAT") then
              estr(jst,jb) = wsel - tsdepth(j,jj)
            else if (estr(jst,jb) > wsel) then
              if (elcontspl(j) == '     OFF') then
                no_flow(j,jj) = .TRUE.                                       ! no flow-- high and dry
                num_noflow    = num_noflow + 1
              else
                estr(jst,jb) = wsel                                          ! poor man's floating outlet
              end if
            end if
            if (.not. no_flow(j,jj) .and. tsminhead(j,jj) > 0.0 .and. wsel - estr(jst,jb) < tsminhead(j,jj)) then
              no_flow(j,jj) = .TRUE.                                         ! minimum head criterion not met -- no flow
              num_noflow    = num_noflow + 1
            end if
            if (.not. no_flow(j,jj) .and. tsmaxhead(j,jj) > 0.0 .and. wsel - estr(jst,jb) > tsmaxhead(j,jj)) then
              no_flow(j,jj) = .TRUE.                                         ! maximum head criterion exceeded -- no flow
              num_noflow    = num_noflow + 1
            end if
            do k=ktwb(jw),kb(ds(jb))
              if (el(k,ds(jb))-elr < estr(jst,jb)) exit
            end do
            kstrsplt(jj)     = min(k-1,kb(ds(jb)))
            qstrfrac(jst,jb) = 0.0                                           ! initialize flow fractions
          end do

        ! Use priority inputs to determine which outlets to use
          prior1 = -999
          do jj=1,nouts(j)
            if (.not. no_flow(j,jj) .and. tsprior(j,jj) >= 0) then
              if (prior1 == -999 .or. tsprior(j,jj) < prior1) prior1 = tsprior(j,jj)
            end if
          end do
          prior2 = -999
          do jj=1,nouts(j)
            if (.not. no_flow(j,jj) .and. tsprior(j,jj) >= 0 .and. tsprior(j,jj) > prior1) then
              if (prior2 == -999 .or. tsprior(j,jj) < prior2) prior2 = tsprior(j,jj)
            end if
          end do

        ! Outlets with a priority of -1 get used, but are not blended
          ng0 = 0
          q_notblended = 0.0
          do jj=1,nouts(j)
            jst = jstsplt(j,jj)
            if (.not. no_flow(j,jj) .and. tsprior(j,jj) == -1) then
              ng0 = ng0 + 1
              nout0(ng0) = jj
              if (qstr(jst,jb) > tsmaxflow(j,jj) .and. tsmaxflow(j,jj) > 0.0) then
                q_notblended = q_notblended + tsmaxflow(j,jj)
                qstrfrac(jst,jb) = tsmaxflow(j,jj) / qall
              else if (qall > 0.0) then
                q_notblended = q_notblended + qstr(jst,jb)
                qstrfrac(jst,jb) = qstr(jst,jb) / qall
              end if
            end if
          end do
          sum_minfrac0 = 0.0
          if (qall > 0.0) sum_minfrac0 = q_notblended / qall

        ! Outlets with priority 1 and 2 may be used and blended.
          ng1 = 0
          ng2 = 0
          sum_minfrac1 = 0.0
          sum_minfrac2 = 0.0
          sum_maxfrac1 = 0.0
          sum_maxfrac2 = 0.0
          do jj=1,nouts(j)
            if (.not. no_flow(j,jj)) then
              if (tsprior(j,jj) == prior1) then
                ng1 = ng1 + 1
                nout1(ng1) = jj
                maxfrac1(ng1) = 1.0
                if (qall > 0.0 .and. tsmaxflow(j,jj) > 0.0) maxfrac1(ng1) = min(1.0, tsmaxflow(j,jj)/qall)
                minfrac1(ng1) = tsminfrac(j,jj)
                if (tsminfrac(j,jj) < 0.0) then
                  minfrac1(ng1) = 0.0
                  if (qall > 0.0) minfrac1(ng1) = min(1.0, abs(tsminfrac(j,jj))/qall)
                end if
                if (minfrac1(ng1) > maxfrac1(ng1)) minfrac1(ng1) = maxfrac1(ng1)
                sum_minfrac1 = sum_minfrac1 + minfrac1(ng1)
                sum_maxfrac1 = sum_maxfrac1 + maxfrac1(ng1)

              else if (tsprior(j,jj) == prior2) then
                ng2 = ng2 + 1
                nout2(ng2) = jj
                maxfrac2(ng2) = 1.0
                if (qall > 0.0 .and. tsmaxflow(j,jj) > 0.0) maxfrac2(ng2) = min(1.0, tsmaxflow(j,jj)/qall)
                minfrac2(ng2) = tsminfrac(j,jj)
                if (tsminfrac(j,jj) < 0.0) then
                  minfrac2(ng2) = 0.0
                  if (qall > 0.0) minfrac2(ng2) = min(1.0, abs(tsminfrac(j,jj))/qall)
                end if
                if (minfrac2(ng2) > maxfrac2(ng2)) minfrac2(ng2) = maxfrac2(ng2)
                sum_minfrac2 = sum_minfrac2 + minfrac2(ng2)
                sum_maxfrac2 = sum_maxfrac2 + maxfrac2(ng2)
              end if
            end if
          end do

        ! If minimum flows are overspecified, then the priority 2 minimum flow fractions are decreased.
          if (ng2 > 0 .and. sum_minfrac0 + sum_minfrac1 + sum_minfrac2 > 1.0) then
            if (sum_minfrac0 + sum_minfrac1 >= 1.0) then
              ng2 = 0
              sum_minfrac2 = 0.0
            else
              do n=1,ng2
                minfrac2(n) = minfrac2(n) * (1.0 - sum_minfrac0 - sum_minfrac1) / sum_minfrac2
              end do
              sum_minfrac2 = 1.0 - sum_minfrac0 - sum_minfrac1
            end if
          end if

        ! If minimum flows are still overspecified, then the priority 1 minimum flow fractions are decreased.
          if (ng1 > 0 .and. sum_minfrac0 + sum_minfrac1 > 1.0) then
            if (sum_minfrac0 >= 1.0) then
              ng1 = 0
              sum_minfrac1 = 0.0
            else
              do n=1,ng1
                minfrac1(n) = minfrac1(n) * (1.0 - sum_minfrac0) / sum_minfrac1
              end do
              sum_minfrac1 = 1.0 - sum_minfrac0
            end if
          end if

        ! If group 1 has 3 or more outlets and group 2 has no outlets, then redistribute priorities based on elevation.
        ! Keep the highest and lowest elevation outlets in group 1, and put other active outlets into nonblended category
        ! with their minimum flows.  If ties in elevation exist, go with the first in the list.
          if (ng1 > 2 .and. ng2 == 0) then
            ng1max  = 1
            ng1min  = 1
            jst     = jstsplt(j,nout1(1))
            maxelev = estr(jst,jb)
            minelev = estr(jst,jb)
            do n=2,ng1
              jst = jstsplt(j,nout1(n))
              if (estr(jst,jb) > maxelev) then
                maxelev = estr(jst,jb)
                ng1max  = n
              else if (estr(jst,jb) < minelev) then
                minelev = estr(jst,jb)
                ng1min  = n
              end if
            end do
            blendfrac = 1.0 - sum_minfrac0 - sum_minfrac1 + minfrac1(ng1max) + minfrac1(ng1min)
            if (maxfrac1(ng1max) + maxfrac1(ng1min) < blendfrac) then
              if (sum_maxfrac1 < 1.0 - sum_minfrac0) then
                write (wrn,'(A,I0,A,F0.3)') 'Warning-- Maximum flows for outlets exceeded for group ', j, ' at day ', jday
                WARNING_OPEN = .TRUE.
                do n=1,ng1
                  if (n .ne. ng1max .and. n .ne. ng1min) minfrac1(n) = maxfrac1(n)
                end do
              else
                excess_frac = blendfrac - maxfrac1(ng1max) - maxfrac1(ng1min)
                num_left = ng1 - 2
                do nj=1,ng1                                        ! iterative process to redistribute excess flows
                  if (num_left > 0 .and. excess_frac > 0.0) then
                    addfrac = excess_frac / num_left
                    do n=1,ng1
                      if (n .ne. ng1max .and. n .ne. ng1min .and. maxfrac1(n) - minfrac1(n) > 0.00001) then
                        if (minfrac1(n) + addfrac > maxfrac1(n)) then
                          num_left    = num_left - 1
                          excess_frac = excess_frac - (maxfrac1(n) - minfrac1(n))
                          minfrac1(n) = maxfrac1(n)
                        else
                          excess_frac = excess_frac - addfrac
                          minfrac1(n) = minfrac1(n) + addfrac
                        end if
                      end if
                    end do
                  end if
                end do
              end if
            end if
            do n=1,ng1                                       ! assign the other priority 1 outlets to nonblended status
              if (n .ne. ng1max .and. n .ne. ng1min) then
                ng0              = ng0 + 1
                nout0(ng0)       = nout1(n)
                jst              = jstsplt(j,nout1(n))
                sum_minfrac0     = sum_minfrac0 + minfrac1(n)
                q_notblended     = q_notblended + qall * minfrac1(n)
                qstrfrac(jst,jb) = minfrac1(n)
              end if
            end do
            ng1 = 1                                ! rearrange outlets-- one in each priority group, but same priority
            ng2 = 1
            nout2(1)     = nout1(ng1min)
            minfrac2(1)  = minfrac1(ng1min)
            maxfrac2(1)  = maxfrac1(ng1min)
            sum_minfrac2 = minfrac1(ng1min)
            sum_maxfrac2 = maxfrac1(ng1min)
            nout1(1)     = nout1(ng1max)
            minfrac1(1)  = minfrac1(ng1max)
            maxfrac1(1)  = maxfrac1(ng1max)
            sum_minfrac1 = minfrac1(ng1max)
            sum_maxfrac1 = maxfrac1(ng1max)
            prior2       = prior1
          end if

        ! If only two blended outlets, ensure that they are in separate groups.
          if (ng1 == 2 .and. ng2 == 0) then
            ng1 = 1
            ng2 = 1
            nout2(1)     = nout1(2)
            minfrac2(1)  = minfrac1(2)
            maxfrac2(1)  = maxfrac1(2)
            sum_minfrac2 = minfrac1(2)
            sum_maxfrac2 = maxfrac1(2)
            sum_minfrac1 = minfrac1(1)
            sum_maxfrac1 = maxfrac1(1)
            prior2       = prior1
          end if


        ! Begin the blending decisions.
        ! No usable outlets.  All flow fractions remain at zero.
          if (nouts(j) == num_noflow) then
            write (wrn,'(A,I0,A,F0.3)') 'Warning-- All outlets dry or unusable for group ', j, ' at day ', jday
            WARNING_OPEN = .TRUE.

        ! Only nonblended outlets.
          else if (nouts(j) == ng0) then
            write (wrn,'(A,I0,A,F0.3)') 'Warning-- Only nonblended outlets present in group ', j, ' at day ', jday
            WARNING_OPEN = .TRUE.

        ! Only one blended outlet.  It gets all of the blended flow, but must not exceed its maximum flow criterion.
          else if (ng1 + ng2 == 1) then
            jst = jstsplt(j,nout1(1))
            qstrfrac(jst,jb) = 1.0 - sum_minfrac0
            if (qall - q_notblended > tsmaxflow(j,nout1(1)) .and. tsmaxflow(j,nout1(1)) > 0.0) then
              qstrfrac(jst,jb) = tsmaxflow(j,nout1(1)) / qall
              write (wrn,'(A,A,I0,A,I0,A,F0.3)') 'Warning-- Total release flow rate decreased to comply with maximum flow ',       &
                                                 'criterion for structure ', jst, ' in group ', j, ' at day ', jday
              WARNING_OPEN = .TRUE.
            end if

        ! Minimum flows comprise entire release.  No blending calculations required.
          else if (abs(1.0 - sum_minfrac0 - sum_minfrac1 - sum_minfrac2) <= 0.000001) then
            do n=1,ng1
              jst = jstsplt(j,nout1(n))
              qstrfrac(jst,jb) = minfrac1(n)
            end do
            do n=1,ng2
              jst = jstsplt(j,nout2(n))
              qstrfrac(jst,jb) = minfrac2(n)
            end do

        ! More than one usable outlet, and blending among priority 1 outlet(s) and priority 2 outlet(s) required.
          else
            id = ds(jb)                                                      ! needed for downstream_withdrawal_estimate
            kt = ktwb(jw)                                                    ! needed for downstream_withdrawal_estimate

          ! Warn the user if maximum flow criteria are likely to decrease the specified outflows.
            if (sum_minfrac0 + sum_maxfrac1 + sum_maxfrac2 < 1.0) then
              write (wrn,'(A,A,I0,A,F0.3)') 'Warning-- Total release flow rate may be decreased to comply with maximum flow ',     &
                                            'criteria for structures in group ', j, ' at day ', jday
              WARNING_OPEN = .TRUE.
            end if

          ! Set the initial release fractions.  Ensure that maximum flows are not exceeded.
            qfrac1 = sum_minfrac1 + 0.5 * (1.0 - sum_minfrac0 - sum_minfrac1 - sum_minfrac2)
            qfrac1 = min(sum_maxfrac1, qfrac1)
            qfrac2 = 1.0 - sum_minfrac0 - qfrac1
            if (qfrac2 > sum_maxfrac2) then
              excess_frac = qfrac2 - sum_maxfrac2
              if (qfrac1 + excess_frac <= sum_maxfrac1) then
                qfrac1 = qfrac1 + excess_frac
              else
                qfrac1 = sum_maxfrac1
              end if
              qfrac2 = sum_maxfrac2
            end if
            j2pref = 1
            call Set_Flow_Fracs(j, jb, j2pref)                               ! set flow fractions; redistribute if maxfrac exceeded

          ! If priority 2 outlets are not sharing flows, identify the two with the highest and lowest elevations
            if (.not. share_flow(j) .and. ng2 > 1) then
              j2hi    = 1
              j2lo    = 1
              jst     = jstsplt(j,nout2(1))
              maxelev = estr(jst,jb)
              minelev = estr(jst,jb)
              do n=2,ng2
                jst = jstsplt(j,nout2(n))
                if (estr(jst,jb) > maxelev) then
                  maxelev = estr(jst,jb)
                  j2hi    = n
                else if (estr(jst,jb) < minelev) then
                  minelev = estr(jst,jb)
                  j2lo    = n
                end if
              end do
            end if

          ! Get weighted blend of all nonblended release temperatures
            ttarg = tspltt(j)
            if (sum_minfrac0 > 0.0) then
              sumtemp = 0.0
              do n=1,ng0
                jst = jstsplt(j,nout0(n))
                qstr(jst,jb) = qall * qstrfrac(jst,jb)
                if (qstr(jst,jb) > 0.0) then
                  call downstream_withdrawal_estimate(jst,etemp,estr(jst,jb))
                else
                  etemp = t2(kstrsplt(nout0(n)),ds(jb))                      ! Use temperature at outlet elevation if no flow
                end if
                sumtemp = sumtemp + qstrfrac(jst,jb) * etemp
              end do
              etemp = sumtemp / sum_minfrac0
              ttarg = (ttarg - sum_minfrac0 * etemp) / (1.0 - sum_minfrac0)  ! New temperature target for blended releases
            end if

          ! Need an iterative approach because released T depends on Q
            lastfrac = qfrac1
            do jj=1,8                                                        ! Maximum of eight iterations
              lastfrac2 = lastfrac
              lastfrac  = qfrac1

              sumtemp = 0.0
              sumelev = 0.0
              do n=1,ng1                                                     ! Get weighted temp and elevation for group 1
                jst = jstsplt(j,nout1(n))
                qstr(jst,jb) = qall * qstrfrac(jst,jb)
                if (qstr(jst,jb) > 0.0) then
                  call downstream_withdrawal_estimate(jst,etemp,estr(jst,jb))
                else
                  etemp = t2(kstrsplt(nout1(n)),ds(jb))                      ! Use temperature at outlet elevation if no flow
                end if
                if (qfrac1 > 0.0) then
                  sumtemp = sumtemp + qstrfrac(jst,jb) * etemp
                  sumelev = sumelev + qstrfrac(jst,jb) * estr(jst,jb)
                else
                  sumtemp = sumtemp + etemp
                  sumelev = sumelev + estr(jst,jb)
                end if
              end do
              if (qfrac1 > 0.0) then
                etemp1 = sumtemp / qfrac1                                    ! Weighted temperature from group 1 outlets
                elev1  = sumelev / qfrac1                                    ! Weighted elevation of group 1 outlets
              else
                etemp1 = sumtemp / ng1
                elev1  = sumelev / ng1
              end if

              if (share_flow(j) .or. ng2 < 2) then                           ! Get weighted temp and elevation for group 2
                sumtemp = 0.0                                                ! ...when flows are shared among outlets
                sumelev = 0.0
                do n=1,ng2
                  jst = jstsplt(j,nout2(n))
                  qstr(jst,jb) = qall * qstrfrac(jst,jb)
                  if (qstr(jst,jb) > 0.0) then
                    call downstream_withdrawal_estimate(jst,etemp,estr(jst,jb))
                  else
                    etemp = t2(kstrsplt(nout2(n)),ds(jb))                    ! Use temperature at outlet elevation if no flow
                  end if
                  if (qfrac2 > 0.0) then
                    sumtemp = sumtemp + qstrfrac(jst,jb) * etemp
                    sumelev = sumelev + qstrfrac(jst,jb) * estr(jst,jb)
                  else
                    sumtemp = sumtemp + etemp
                    sumelev = sumelev + estr(jst,jb)
                  end if
                end do
                if (qfrac2 > 0.0) then
                  etemp2 = sumtemp / qfrac2                                  ! Weighted temperature from group 2 outlets
                  elev2  = sumelev / qfrac2                                  ! Weighted elevation of group 2 outlets
                else
                  etemp2 = sumtemp / ng2
                  elev2  = sumelev / ng2
                end if

              else                                                           ! ...and when flows are not shared
                if (qfrac2 == 0.0) then
                  do n=1,ng2
                    jst       = jstsplt(j,nout2(n))
                    splt2t(n) = t2(kstrsplt(nout2(n)),ds(jb))
                    splt2e(n) = estr(jst,jb)
                  end do
                else
                  do nj=1,ng2                                                ! Find the temperatures produced in group 2
                    sumtemp = 0.0                                            ! by testing when each outlet is preferred
                    sumelev = 0.0
                    call Set_Flow_Fracs2(j, jb, nj)
                    do n=1,ng2
                      jst = jstsplt(j,nout2(n))
                      qstr(jst,jb) = qall * qstrfrac(jst,jb)
                      if (qstr(jst,jb) > 0.0) then
                        call downstream_withdrawal_estimate(jst,etemp,estr(jst,jb))
                      else
                        etemp = t2(kstrsplt(nout2(n)),ds(jb))
                      end if
                      sumtemp = sumtemp + qstrfrac(jst,jb) * etemp
                      sumelev = sumelev + qstrfrac(jst,jb) * estr(jst,jb)
                    end do
                    splt2t(nj) = sumtemp / qfrac2
                    splt2e(nj) = sumelev / qfrac2
                  end do
                end if
                j2max   = 1
                j2min   = 1
                maxtemp = splt2t(1)
                mintemp = splt2t(1)
                do n=2,ng2
                  if (splt2t(n) > maxtemp) then
                    maxtemp = splt2t(n)
                    j2max   = n
                  else if (splt2t(n) < mintemp) then
                    mintemp = splt2t(n)
                    j2min   = n
                  end if
                end do
                if (ttarg < etemp1 - 0.001) then                             ! need a colder temp from group 2
                  if (maxtemp - mintemp > 0.001) then
                    etemp2 = splt2t(j2min)                                   ! preferred outlet is the coldest one
                    elev2  = splt2e(j2min)
                    j2pref = j2min
                  else
                    etemp2 = splt2t(j2lo)                                    ! preferred outlet is the lowest one
                    elev2  = splt2e(j2lo)
                    j2pref = j2lo
                  end if
                else if (ttarg > etemp1 + 0.001) then                        ! need a warmer temp from group 2
                  if (maxtemp - mintemp > 0.001) then
                    etemp2 = splt2t(j2max)                                   ! preferred outlet is the warmest one
                    elev2  = splt2e(j2max)
                    j2pref = j2max
                  else
                    etemp2 = splt2t(j2hi)                                    ! preferred outlet is the highest one
                    elev2  = splt2e(j2hi)
                    j2pref = j2hi
                  end if
                else
                  etemp2 = splt2t(1)                                         ! if temp is close to target, choose first outlet
                  elev2  = splt2e(1)
                  j2pref = 1
                end if
              end if

            ! Target temperature is less than either outlet temperature.
              if (ttarg < etemp1 .and. ttarg < etemp2) then
                qfrac1 = sum_minfrac1                                        ! default for if/then cases
                if (abs(etemp1 - etemp2) < 0.001) then
                  if (prior1 == prior2) then                                 ! If each outlet has the same priority level, then...
                    if (elev1 <= elev2) then                                 ! Choose lower outlet if both have same temperature.
                      qfrac1 = 1.0 - sum_minfrac0 - sum_minfrac2
                    end if
                  else if (prior1 < prior2) then
                    qfrac1 = 1.0 - sum_minfrac0 - sum_minfrac2               ! Choose higher priority outlet if temps are the same.
                  end if
                else if (etemp1 < etemp2) then                               ! If temps are different, choose the one closer
                  qfrac1 = 1.0 - sum_minfrac0 - sum_minfrac2                 ! to target temperature
                end if

            ! Target temperature is greater than either outlet temperature.
              else if (ttarg > etemp1 .and. ttarg > etemp2) then
                qfrac1 = sum_minfrac1                                        ! default for if/then cases
                if (abs(etemp1 - etemp2) < 0.001) then
                  if (prior1 == prior2) then                                 ! If each outlet has the same priority level, then...
                    if (elev1 >= elev2) then                                 ! Choose upper outlet if both have same temperature.
                      qfrac1 = 1.0 - sum_minfrac0 - sum_minfrac2
                    end if
                  else if (prior1 < prior2) then
                    qfrac1 = 1.0 - sum_minfrac0 - sum_minfrac2               ! Choose higher priority outlet if temps are the same.
                  end if
                else if (etemp1 > etemp2) then                               ! If temps are different, choose the one closer
                  qfrac1 = 1.0 - sum_minfrac0 - sum_minfrac2                 ! to target temperature
                end if

            ! Target temperature is essentially the same as the two outlet temperatures.
              else if (abs(etemp1 - etemp2) < 0.001) then
                qfrac1 = sum_minfrac1                                        ! default for if/then cases
                if (prior1 == prior2) then                                   ! If each outlet has the same priority level, then...
                  qfrac1 = sum_minfrac1 + 0.5 * (1.0 - sum_minfrac0 - sum_minfrac1 - sum_minfrac2)    ! Split the flow equally.
                else if (prior1 < prior2) then
                  qfrac1 = 1.0 - sum_minfrac0 - sum_minfrac2                 ! Choose higher priority outlet if temps are the same.
                end if

            ! Target temperature is between the two outlet temperatures.
              else
                qfrac1 = (1.0 - sum_minfrac0) * abs((ttarg-etemp2)/(etemp1-etemp2+NONZERO))
                qfrac1 = max(sum_minfrac1, qfrac1)
                qfrac1 = min(1.0 - sum_minfrac0 - sum_minfrac2, qfrac1)
              end if
              qfrac1 = min(sum_maxfrac1, qfrac1)
              qfrac2 = 1.0 - sum_minfrac0 - qfrac1
              if (qfrac2 > sum_maxfrac2) then
                excess_frac = qfrac2 - sum_maxfrac2
                if (qfrac1 + excess_frac <= sum_maxfrac1) then
                  qfrac1 = qfrac1 + excess_frac
                else
                  qfrac1 = sum_maxfrac1
                end if
                qfrac2 = sum_maxfrac2
              end if

            ! Set flow fractions for individual outlets and redistribute flows if maximum flow fractions exceeded.
              call Set_Flow_Fracs(j, jb, j2pref)

            ! Exit the loop if the latest flow fraction calculation agrees with the previous one.
            ! Exit if no flow, because no iteration requried in that case.
              if (abs(lastfrac - qfrac1) < tsconv .or. qall == 0.0) exit
            end do

          ! Check to see if iterative solution did not converge.
            if (abs(lastfrac - qfrac1) >= tsconv .and. qall > 0.0) then
              write (wrn,'(A,F0.3,3(A,F0.4))') 'Flow fraction calculations not converging at day ', jday,                          &
                                               '  Current: ', qfrac1, ' Last: ', lastfrac, ' Next-to-last: ', lastfrac2
              WARNING_OPEN = .TRUE.

            ! Check to see if the iterative solution is unstable.  If so, use priorities to assign releases.
            ! Criteria:  change is at least 0.1 and most recent change is in opposite direction from previous change.
              if (abs(lastfrac - qfrac1) >= 0.1 .and. (qfrac1-lastfrac)*(lastfrac-lastfrac2) < 0.0) then
                qfrac1 = sum_minfrac1                                        ! default for if/then cases
                if (prior1 < prior2) then                                    ! group 1 is higher priority
                  qfrac1 = 1.0 - sum_minfrac0 - sum_minfrac2
                else                                                         ! else, fulfill minima and split the rest
                  qfrac1 = sum_minfrac1 + 0.5 * (1.0 - sum_minfrac0 - sum_minfrac1 - sum_minfrac2)
                end if
                qfrac1 = min(sum_maxfrac1, qfrac1)
                qfrac2 = 1.0 - sum_minfrac0 - qfrac1
                if (qfrac2 > sum_maxfrac2) then
                  excess_frac = qfrac2 - sum_maxfrac2
                  if (qfrac1 + excess_frac <= sum_maxfrac1) then
                    qfrac1 = qfrac1 + excess_frac
                  else
                    qfrac1 = sum_maxfrac1
                  end if
                  qfrac2 = sum_maxfrac2
                end if
                call Set_Flow_Fracs(j, jb, j2pref)                           ! set flow fractions; redistribute if maxfrac exceeded
              end if
            end if
          end if

        ! Set final flows to go with the flow fractions.  May not be necessary, but do it anyway.
          do jj=1,nouts(j)
            qstr(jstsplt(j,jj),jb) = qall * qstrfrac(jstsplt(j,jj),jb)
          end do


      ! Do Withdrawals next
        else if (tspltcntr(j) == '      WD') then
          jwd = jstsplt(j,1)                                                 ! assume withdrawals are from same branch and waterbody
          do jb=1,nbr
            if (iwd(jwd) >= us(jb) .and. iwd(jwd) <= ds(jb)) exit
          end do
          do jw=1,nwb
            if (jb >= bs(jw) .and. jb <= be(jw)) exit
          end do
          no_flow(j,:) = .FALSE.
          num_noflow   = 0
          do jj=1,nouts(j)
            jwd      = jstsplt(j,jj)
            elr      = sina(jb) * dlx(iwd(jwd)) * 0.5
            wsel     = elws(iwd(jwd)) - elr                                  ! compute water-surface elevation
            ewd(jwd) = ewdsav(jwd)                                           ! reset outlet elevation to original
            if (tstype(j,jj) .eq. "FLOAT") then
              ewd(jwd) = wsel - tsdepth(j,jj)
            else if (ewd(jwd) > wsel) then
              if (elcontspl(j) == '     OFF') then
                no_flow(j,jj) = .TRUE.                                       ! no flow-- high and dry
                num_noflow    = num_noflow + 1
              else
                ewd(jwd) = wsel                                              ! poor man's floating outlet
              end if
            end if
            if (.not. no_flow(j,jj) .and. tsminhead(j,jj) > 0.0 .and. wsel - ewd(jwd) < tsminhead(j,jj)) then
              no_flow(j,jj) = .TRUE.                                         ! minimum head criterion not met -- no flow
              num_noflow    = num_noflow + 1
            end if
            if (.not. no_flow(j,jj) .and. tsmaxhead(j,jj) > 0.0 .and. wsel - ewd(jwd) > tsmaxhead(j,jj)) then
              no_flow(j,jj) = .TRUE.                                         ! maximum head criterion exceeded -- no flow
              num_noflow    = num_noflow + 1
            end if
            do k=ktwb(jw),kb(iwd(jwd))
              if (el(k,iwd(jwd))-elr < ewd(jwd)) exit
            end do
            kstrsplt(jj) = min(k-1,kb(iwd(jwd)))
            qwdfrac(jwd) = 0.0                                               ! initialize flow fractions
          end do

        ! Use priority inputs to determine which outlets to use
          prior1 = -999
          do jj=1,nouts(j)
            if (.not. no_flow(j,jj) .and. tsprior(j,jj) >= 0) then
              if (prior1 == -999 .or. tsprior(j,jj) < prior1) prior1 = tsprior(j,jj)
            end if
          end do
          prior2 = -999
          do jj=1,nouts(j)
            if (.not. no_flow(j,jj) .and. tsprior(j,jj) >= 0 .and. tsprior(j,jj) > prior1) then
              if (prior2 == -999 .or. tsprior(j,jj) < prior2) prior2 = tsprior(j,jj)
            end if
          end do

        ! Outlets with a priority of -1 get used, but are not blended
          ng0 = 0
          q_notblended = 0.0
          do jj=1,nouts(j)
            jwd = jstsplt(j,jj)
            if (.not. no_flow(j,jj) .and. tsprior(j,jj) == -1) then
              ng0 = ng0 + 1
              nout0(ng0) = jj
              if (qwd(jwd) > tsmaxflow(j,jj) .and. tsmaxflow(j,jj) > 0.0) then
                q_notblended = q_notblended + tsmaxflow(j,jj)
                qwdfrac(jwd) = tsmaxflow(j,jj) / qall
              else if (qall > 0.0) then
                q_notblended = q_notblended + qwd(jwd)
                qwdfrac(jwd) = qwd(jwd) / qall
              end if
            end if
          end do
          sum_minfrac0 = 0.0
          if (qall > 0.0) sum_minfrac0 = q_notblended / qall

        ! Outlets with priority 1 and 2 may be used and blended.
          ng1 = 0
          ng2 = 0
          sum_minfrac1 = 0.0
          sum_minfrac2 = 0.0
          sum_maxfrac1 = 0.0
          sum_maxfrac2 = 0.0
          do jj=1,nouts(j)
            if (.not. no_flow(j,jj)) then
              if (tsprior(j,jj) == prior1) then
                ng1 = ng1 + 1
                nout1(ng1) = jj
                maxfrac1(ng1) = 1.0
                if (qall > 0.0 .and. tsmaxflow(j,jj) > 0.0) maxfrac1(ng1) = min(1.0, tsmaxflow(j,jj)/qall)
                minfrac1(ng1) = tsminfrac(j,jj)
                if (tsminfrac(j,jj) < 0.0) then
                  minfrac1(ng1) = 0.0
                  if (qall > 0.0) minfrac1(ng1) = min(1.0, abs(tsminfrac(j,jj))/qall)
                end if
                if (minfrac1(ng1) > maxfrac1(ng1)) minfrac1(ng1) = maxfrac1(ng1)
                sum_minfrac1 = sum_minfrac1 + minfrac1(ng1)
                sum_maxfrac1 = sum_maxfrac1 + maxfrac1(ng1)

              else if (tsprior(j,jj) == prior2) then
                ng2 = ng2 + 1
                nout2(ng2) = jj
                maxfrac2(ng2) = 1.0
                if (qall > 0.0 .and. tsmaxflow(j,jj) > 0.0) maxfrac2(ng2) = min(1.0, tsmaxflow(j,jj)/qall)
                minfrac2(ng2) = tsminfrac(j,jj)
                if (tsminfrac(j,jj) < 0.0) then
                  minfrac2(ng2) = 0.0
                  if (qall > 0.0) minfrac2(ng2) = min(1.0, abs(tsminfrac(j,jj))/qall)
                end if
                if (minfrac2(ng2) > maxfrac2(ng2)) minfrac2(ng2) = maxfrac2(ng2)
                sum_minfrac2 = sum_minfrac2 + minfrac2(ng2)
                sum_maxfrac2 = sum_maxfrac2 + maxfrac2(ng2)
              end if
            end if
          end do

        ! If minimum flows are overspecified, then the priority 2 minimum flow fractions are decreased.
          if (ng2 > 0 .and. sum_minfrac0 + sum_minfrac1 + sum_minfrac2 > 1.0) then
            if (sum_minfrac0 + sum_minfrac1 >= 1.0) then
              ng2 = 0
              sum_minfrac2 = 0.0
            else
              do n=1,ng2
                minfrac2(n) = minfrac2(n) * (1.0 - sum_minfrac0 - sum_minfrac1) / sum_minfrac2
              end do
              sum_minfrac2 = 1.0 - sum_minfrac0 - sum_minfrac1
            end if
          end if

        ! If minimum flows are still overspecified, then the priority 1 minimum flow fractions are decreased.
          if (ng1 > 0 .and. sum_minfrac0 + sum_minfrac1 > 1.0) then
            if (sum_minfrac0 >= 1.0) then
              ng1 = 0
              sum_minfrac1 = 0.0
            else
              do n=1,ng1
                minfrac1(n) = minfrac1(n) * (1.0 - sum_minfrac0) / sum_minfrac1
              end do
              sum_minfrac1 = 1.0 - sum_minfrac0
            end if
          end if

        ! If group 1 has 3 or more outlets and group 2 has no outlets, then redistribute priorities based on elevation.
        ! Keep the highest and lowest elevation outlets in group 1, and put other active outlets into nonblended category
        ! with their minimum flows.  If ties in elevation exist, go with the first in the list.
          if (ng1 > 2 .and. ng2 == 0) then
            ng1max  = 1
            ng1min  = 1
            jwd     = jstsplt(j,nout1(1))
            maxelev = ewd(jwd)
            minelev = ewd(jwd)
            do n=2,ng1
              jwd = jstsplt(j,nout1(n))
              if (ewd(jwd) > maxelev) then
                maxelev = ewd(jwd)
                ng1max  = n
              else if (ewd(jwd) < minelev) then
                minelev = ewd(jwd)
                ng1min  = n
              end if
            end do
            blendfrac = 1.0 - sum_minfrac0 - sum_minfrac1 + minfrac1(ng1max) + minfrac1(ng1min)
            if (maxfrac1(ng1max) + maxfrac1(ng1min) < blendfrac) then
              if (sum_maxfrac1 < 1.0 - sum_minfrac0) then
                write (wrn,'(A,I0,A,F0.3)') 'Warning-- Maximum flows for outlets exceeded for group ', j, ' at day ', jday
                WARNING_OPEN = .TRUE.
                do n=1,ng1
                  if (n .ne. ng1max .and. n .ne. ng1min) minfrac1(n) = maxfrac1(n)
                end do
              else
                excess_frac = blendfrac - maxfrac1(ng1max) - maxfrac1(ng1min)
                num_left = ng1 - 2
                do nj=1,ng1                                        ! iterative process to redistribute excess flows
                  if (num_left > 0 .and. excess_frac > 0.0) then
                    addfrac = excess_frac / num_left
                    do n=1,ng1
                      if (n .ne. ng1max .and. n .ne. ng1min .and. maxfrac1(n) - minfrac1(n) > 0.00001) then
                        if (minfrac1(n) + addfrac > maxfrac1(n)) then
                          num_left    = num_left - 1
                          excess_frac = excess_frac - (maxfrac1(n) - minfrac1(n))
                          minfrac1(n) = maxfrac1(n)
                        else
                          excess_frac = excess_frac - addfrac
                          minfrac1(n) = minfrac1(n) + addfrac
                        end if
                      end if
                    end do
                  end if
                end do
              end if
            end if
            do n=1,ng1                                       ! assign the other priority 1 outlets to nonblended status
              if (n .ne. ng1max .and. n .ne. ng1min) then
                ng0          = ng0 + 1
                nout0(ng0)   = nout1(n)
                jwd          = jstsplt(j,nout1(n))
                sum_minfrac0 = sum_minfrac0 + minfrac1(n)
                q_notblended = q_notblended + qall * minfrac1(n)
                qwdfrac(jwd) = minfrac1(n)
              end if
            end do
            ng1 = 1                                ! rearrange outlets-- one in each priority group, but same priority
            ng2 = 1
            nout2(1)     = nout1(ng1min)
            minfrac2(1)  = minfrac1(ng1min)
            maxfrac2(1)  = maxfrac1(ng1min)
            sum_minfrac2 = minfrac1(ng1min)
            sum_maxfrac2 = maxfrac1(ng1min)
            nout1(1)     = nout1(ng1max)
            minfrac1(1)  = minfrac1(ng1max)
            maxfrac1(1)  = maxfrac1(ng1max)
            sum_minfrac1 = minfrac1(ng1max)
            sum_maxfrac1 = maxfrac1(ng1max)
            prior2       = prior1
          end if

        ! If only two blended outlets, ensure that they are in separate groups.
          if (ng1 == 2 .and. ng2 == 0) then
            ng1 = 1
            ng2 = 1
            nout2(1)     = nout1(2)
            minfrac2(1)  = minfrac1(2)
            maxfrac2(1)  = maxfrac1(2)
            sum_minfrac2 = minfrac1(2)
            sum_maxfrac2 = maxfrac1(2)
            sum_minfrac1 = minfrac1(1)
            sum_maxfrac1 = maxfrac1(1)
            prior2       = prior1
          end if


        ! Begin the blending decisions.
        ! No usable outlets.  All flow fractions remain at zero.
          if (nouts(j) == num_noflow) then
            write (wrn,'(A,I0,A,F0.3)') 'Warning-- All outlets dry or unusable for group ', j, ' at day ', jday
            WARNING_OPEN = .TRUE.

        ! Only nonblended outlets.
          else if (nouts(j) == ng0) then
            write (wrn,'(A,I0,A,F0.3)') 'Warning-- Only nonblended outlets present in group ', j, ' at day ', jday
            WARNING_OPEN = .TRUE.

        ! Only one blended outlet.  It gets all of the blended flow, but must not exceed its maximum flow criterion.
          else if (ng1 + ng2 == 1) then
            jwd = jstsplt(j,nout1(1))
            qwdfrac(jwd) = 1.0 - sum_minfrac0
            if (qall - q_notblended > tsmaxflow(j,nout1(1)) .and. tsmaxflow(j,nout1(1)) > 0.0) then
              qwdfrac(jwd) = tsmaxflow(j,nout1(1)) / qall
              write (wrn,'(A,A,I0,A,I0,A,F0.3)') 'Warning-- Total release flow rate decreased to comply with maximum flow ',       &
                                                 'criterion for withdrawal ', jwd, ' in group ', j, ' at day ', jday
              WARNING_OPEN = .TRUE.
            end if

        ! Minimum flows comprise entire release.  No blending calculations required.
          else if (abs(1.0 - sum_minfrac0 - sum_minfrac1 - sum_minfrac2) <= 0.000001) then
            do n=1,ng1
              jwd = jstsplt(j,nout1(n))
              qwdfrac(jwd) = minfrac1(n)
            end do
            do n=1,ng2
              jwd = jstsplt(j,nout2(n))
              qwdfrac(jwd) = minfrac2(n)
            end do

        ! More than one usable outlet, and blending among priority 1 outlet(s) and priority 2 outlet(s) required.
          else
            I  = iwd(jstsplt(j,nout1(1)))                                    ! needed for lateral_withdrawal_estimate
            kt = ktwb(jw)                                                    ! needed for lateral_withdrawal_estimate

          ! Warn the user if maximum flow criteria are likely to decrease the specified outflows.
            if (sum_minfrac0 + sum_maxfrac1 + sum_maxfrac2 < 1.0) then
              write (wrn,'(A,A,I0,A,F0.3)') 'Warning-- Total release flow rate may be decreased to comply with maximum flow ',     &
                                            'criteria for withdrawals in group ', j, ' at day ', jday
              WARNING_OPEN = .TRUE.
            end if

          ! Set the initial release fractions.  Ensure that maximum flows are not exceeded.
            qfrac1 = sum_minfrac1 + 0.5 * (1.0 - sum_minfrac0 - sum_minfrac1 - sum_minfrac2)
            qfrac1 = min(sum_maxfrac1, qfrac1)
            qfrac2 = 1.0 - sum_minfrac0 - qfrac1
            if (qfrac2 > sum_maxfrac2) then
              excess_frac = qfrac2 - sum_maxfrac2
              if (qfrac1 + excess_frac <= sum_maxfrac1) then
                qfrac1 = qfrac1 + excess_frac
              else
                qfrac1 = sum_maxfrac1
              end if
              qfrac2 = sum_maxfrac2
            end if
            j2pref = 1
            call Set_Flow_Fracs(j, jb, j2pref)                               ! set flow fractions; redistribute if maxfrac exceeded

          ! If priority 2 outlets are not sharing flows, identify the two with the highest and lowest elevations
            if (.not. share_flow(j) .and. ng2 > 1) then
              j2hi    = 1
              j2lo    = 1
              jwd     = jstsplt(j,nout2(1))
              maxelev = ewd(jwd)
              minelev = ewd(jwd)
              do n=2,ng2
                jwd = jstsplt(j,nout2(n))
                if (ewd(jwd) > maxelev) then
                  maxelev = ewd(jwd)
                  j2hi    = n
                else if (ewd(jwd) < minelev) then
                  minelev = ewd(jwd)
                  j2lo    = n
                end if
              end do
            end if

          ! Get weighted blend of all nonblended release temperatures
            ttarg = tspltt(j)
            if (sum_minfrac0 > 0.0) then
              sumtemp = 0.0
              do n=1,ng0
                jwd = jstsplt(j,nout0(n))
                qwd(jwd) = qall * qwdfrac(jwd)
                if (qwd(jwd) > 0.0) then
                  call lateral_withdrawal_estimate(jwd,etemp,ewd(jwd))       ! Get an estimate of the temperature of the outflow
                else
                  etemp = t2(kstrsplt(nout0(n)),iwd(jwd))                    ! Use temperature at outlet elevation if no flow
                end if
                sumtemp = sumtemp + qwdfrac(jwd) * etemp
              end do
              etemp = sumtemp / sum_minfrac0
              ttarg = (ttarg - sum_minfrac0 * etemp) / (1.0 - sum_minfrac0)  ! New temperature target for blended releases
            end if

          ! Need an iterative approach because released T depends on Q
            lastfrac = qfrac1
            do jj=1,8                                                        ! Maximum of eight iterations
              lastfrac2 = lastfrac
              lastfrac  = qfrac1

              sumtemp = 0.0
              sumelev = 0.0
              do n=1,ng1                                                     ! Get weighted temp and elevation for group 1
                jwd = jstsplt(j,nout1(n))
                qwd(jwd) = qall * qwdfrac(jwd)
                if (qwd(jwd) > 0.0) then
                  call lateral_withdrawal_estimate(jwd,etemp,ewd(jwd))
                else
                  etemp = t2(kstrsplt(nout1(n)),iwd(jwd))                    ! Use temperature at outlet elevation if no flow
                end if
                if (qfrac1 > 0.0) then
                  sumtemp = sumtemp + qwdfrac(jwd) * etemp
                  sumelev = sumelev + qwdfrac(jwd) * ewd(jwd)
                else
                  sumtemp = sumtemp + etemp
                  sumelev = sumelev + ewd(jwd)
                end if
              end do
              if (qfrac1 > 0.0) then
                etemp1 = sumtemp / qfrac1                                    ! Weighted temperature from group 1 outlets
                elev1  = sumelev / qfrac1                                    ! Weighted elevation of group 1 outlets
              else
                etemp1 = sumtemp / ng1
                elev1  = sumelev / ng1
              end if

              if (share_flow(j) .or. ng2 < 2) then                           ! Get weighted temp and elevation for group 2
                sumtemp = 0.0                                                ! ...when flows are shared among outlets
                sumelev = 0.0
                do n=1,ng2
                  jwd = jstsplt(j,nout2(n))
                  qwd(jwd) = qall * qwdfrac(jwd)
                  if (qwd(jwd) > 0.0) then
                    call lateral_withdrawal_estimate(jwd,etemp,ewd(jwd))
                  else
                    etemp = t2(kstrsplt(nout2(n)),iwd(jwd))                  ! Use temperature at outlet elevation if no flow
                  end if
                  if (qfrac2 > 0.0) then
                    sumtemp = sumtemp + qwdfrac(jwd) * etemp
                    sumelev = sumelev + qwdfrac(jwd) * ewd(jwd)
                  else
                    sumtemp = sumtemp + etemp
                    sumelev = sumelev + ewd(jwd)
                  end if
                end do
                if (qfrac2 > 0.0) then
                  etemp2 = sumtemp / qfrac2                                  ! Weighted temperature from group 2 outlets
                  elev2  = sumelev / qfrac2                                  ! Weighted elevation of group 2 outlets
                else
                  etemp2 = sumtemp / ng2
                  elev2  = sumelev / ng2
                end if

              else                                                           ! ...and when flows are not shared
                if (qfrac2 == 0.0) then
                  do n=1,ng2
                    jwd       = jstsplt(j,nout2(n))
                    splt2t(n) = t2(kstrsplt(nout2(n)),iwd(jwd))
                    splt2e(n) = ewd(jwd)
                  end do
                else
                  do nj=1,ng2                                                ! Find the temperatures produced in group 2
                    sumtemp = 0.0                                            ! by testing when each outlet is preferred
                    sumelev = 0.0
                    call Set_Flow_Fracs2(j, jb, nj)
                    do n=1,ng2
                      jwd = jstsplt(j,nout2(n))
                      qwd(jwd) = qall * qwdfrac(jwd)
                      if (qwd(jwd) > 0.0) then
                        call lateral_withdrawal_estimate(jwd,etemp,ewd(jwd))
                      else
                        etemp = t2(kstrsplt(nout2(n)),iwd(jwd))
                      end if
                      sumtemp = sumtemp + qwdfrac(jwd) * etemp
                      sumelev = sumelev + qwdfrac(jwd) * ewd(jwd)
                    end do
                    splt2t(nj) = sumtemp / qfrac2
                    splt2e(nj) = sumelev / qfrac2
                  end do
                end if
                j2max   = 1
                j2min   = 1
                maxtemp = splt2t(1)
                mintemp = splt2t(1)
                do n=2,ng2
                  if (splt2t(n) > maxtemp) then
                    maxtemp = splt2t(n)
                    j2max   = n
                  else if (splt2t(n) < mintemp) then
                    mintemp = splt2t(n)
                    j2min   = n
                  end if
                end do
                if (ttarg < etemp1 - 0.001) then                             ! need a colder temp from group 2
                  if (maxtemp - mintemp > 0.001) then
                    etemp2 = splt2t(j2min)                                   ! preferred outlet is the coldest one
                    elev2  = splt2e(j2min)
                    j2pref = j2min
                  else
                    etemp2 = splt2t(j2lo)                                    ! preferred outlet is the lowest one
                    elev2  = splt2e(j2lo)
                    j2pref = j2lo
                  end if
                else if (ttarg > etemp1 + 0.001) then                        ! need a warmer temp from group 2
                  if (maxtemp - mintemp > 0.001) then
                    etemp2 = splt2t(j2max)                                   ! preferred outlet is the warmest one
                    elev2  = splt2e(j2max)
                    j2pref = j2max
                  else
                    etemp2 = splt2t(j2hi)                                    ! preferred outlet is the highest one
                    elev2  = splt2e(j2hi)
                    j2pref = j2hi
                  end if
                else
                  etemp2 = splt2t(1)                                         ! if temp is close to target, choose first outlet
                  elev2  = splt2e(1)
                  j2pref = 1
                end if
              end if

            ! Target temperature is less than either outlet temperature.
              if (ttarg < etemp1 .and. ttarg < etemp2) then
                qfrac1 = sum_minfrac1                                        ! default for if/then cases
                if (abs(etemp1 - etemp2) < 0.001) then
                  if (prior1 == prior2) then                                 ! If each outlet has the same priority level, then...
                    if (elev1 <= elev2) then                                 ! Choose lower outlet if both have same temperature.
                      qfrac1 = 1.0 - sum_minfrac0 - sum_minfrac2
                    end if
                  else if (prior1 < prior2) then
                    qfrac1 = 1.0 - sum_minfrac0 - sum_minfrac2               ! Choose higher priority outlet if temps are the same.
                  end if
                else if (etemp1 < etemp2) then                               ! If temps are different, choose the one closer
                  qfrac1 = 1.0 - sum_minfrac0 - sum_minfrac2                 ! to target temperature
                end if

            ! Target temperature is greater than either outlet temperature.
              else if (ttarg > etemp1 .and. ttarg > etemp2) then
                qfrac1 = sum_minfrac1                                        ! default for if/then cases
                if (abs(etemp1 - etemp2) < 0.001) then
                  if (prior1 == prior2) then                                 ! If each outlet has the same priority level, then...
                    if (elev1 >= elev2) then                                 ! Choose upper outlet if both have same temperature.
                      qfrac1 = 1.0 - sum_minfrac0 - sum_minfrac2
                    end if
                  else if (prior1 < prior2) then
                    qfrac1 = 1.0 - sum_minfrac0 - sum_minfrac2               ! Choose higher priority outlet if temps are the same.
                  end if
                else if (etemp1 > etemp2) then                               ! If temps are different, choose the one closer
                  qfrac1 = 1.0 - sum_minfrac0 - sum_minfrac2                 ! to target temperature
                end if

            ! Target temperature is essentially the same as the two outlet temperatures.
              else if (abs(etemp1 - etemp2) < 0.001) then
                qfrac1 = sum_minfrac1                                        ! default for if/then cases
                if (prior1 == prior2) then                                   ! If each outlet has the same priority level, then...
                  qfrac1 = sum_minfrac1 + 0.5 * (1.0 - sum_minfrac0 - sum_minfrac1 - sum_minfrac2)    ! Split the flow equally.
                else if (prior1 < prior2) then
                  qfrac1 = 1.0 - sum_minfrac0 - sum_minfrac2                 ! Choose higher priority outlet if temps are the same.
                end if

            ! Target temperature is between the two outlet temperatures.
              else
                qfrac1 = (1.0 - sum_minfrac0) * abs((ttarg-etemp2)/(etemp1-etemp2+NONZERO))
                qfrac1 = max(sum_minfrac1, qfrac1)
                qfrac1 = min(1.0 - sum_minfrac0 - sum_minfrac2, qfrac1)
              end if
              qfrac1 = min(sum_maxfrac1, qfrac1)
              qfrac2 = 1.0 - sum_minfrac0 - qfrac1
              if (qfrac2 > sum_maxfrac2) then
                excess_frac = qfrac2 - sum_maxfrac2
                if (qfrac1 + excess_frac <= sum_maxfrac1) then
                  qfrac1 = qfrac1 + excess_frac
                else
                  qfrac1 = sum_maxfrac1
                end if
                qfrac2 = sum_maxfrac2
              end if

            ! Set flow fractions for individual outlets and redistribute flows if maximum flow fractions exceeded.
              call Set_Flow_Fracs(j, jb, j2pref)

            ! Exit the loop if the latest flow fraction calculation agrees with the previous one.
            ! Exit if no flow, because no iteration requried in that case.
              if (abs(lastfrac - qfrac1) < tsconv .or. qall == 0.0) exit
            end do

          ! Check to see if iterative solution did not converge.
            if (abs(lastfrac - qfrac1) >= tsconv .and. qall > 0.0) then
              write (wrn,'(A,F0.3,3(A,F0.4))') 'Flow fraction calculations not converging at day ', jday,                          &
                                               '  Current: ', qfrac1, ' Last: ', lastfrac, ' Next-to-last: ', lastfrac2
              WARNING_OPEN = .TRUE.

            ! Check to see if the iterative solution is unstable.  If so, use priorities to assign releases.
            ! Criteria:  change is at least 0.1 and most recent change is in opposite direction from previous change.
              if (abs(lastfrac - qfrac1) >= 0.1 .and. (qfrac1-lastfrac)*(lastfrac-lastfrac2) < 0.0) then
                qfrac1 = sum_minfrac1                                        ! default for if/then cases
                if (prior1 < prior2) then                                    ! group 1 is higher priority
                  qfrac1 = 1.0 - sum_minfrac0 - sum_minfrac2
                else                                                         ! else, fulfill minima and split the rest
                  qfrac1 = sum_minfrac1 + 0.5 * (1.0 - sum_minfrac0 - sum_minfrac1 - sum_minfrac2)
                end if
                qfrac1 = min(sum_maxfrac1, qfrac1)
                qfrac2 = 1.0 - sum_minfrac0 - qfrac1
                if (qfrac2 > sum_maxfrac2) then
                  excess_frac = qfrac2 - sum_maxfrac2
                  if (qfrac1 + excess_frac <= sum_maxfrac1) then
                    qfrac1 = qfrac1 + excess_frac
                  else
                    qfrac1 = sum_maxfrac1
                  end if
                  qfrac2 = sum_maxfrac2
                end if
                call Set_Flow_Fracs(j, jb, j2pref)                           ! set flow fractions; redistribute if maxfrac exceeded
              end if
            end if
          end if

        ! Set final flows to go with the flow fractions.  May not be necessary, but do it anyway.
          do jj=1,nouts(j)
            qwd(jstsplt(j,jj)) = qall * qwdfrac(jstsplt(j,jj))
          end do
        end if
      end if
    end do

    nxtsplit = nxtsplit + tspltfreq
  end if

! Use the flow fractions to set flows in blended groups.
  if (tspltc=='      ON') then
    do j=1,numtsplt
      if (tsyearly(j) == '     OFF') then
        daytest = jday
      else
        daytest = real(jdayg) + jday - int(jday)
      end if
      if (daytest >= tstsrt(j) .and. daytest < tstend(j)) then
        qall = 0.0

      ! Do structures first
        if (tspltcntr(j) == '      ST') then
          do jj=1,nouts(j)
            qall = qall + qstr(jstsplt(j,jj),tspltjb(j))                            ! sum up all the flows
          end do
          do jj=1,nouts(j)                                                          ! set the flows and honor the maximum flow
            jst = jstsplt(j,jj)
            qstr(jst,tspltjb(j)) = qstrfrac(jst,tspltjb(j)) * qall
            if (tsmaxflow(j,jj) > 0.0 .and. qstr(jst,tspltjb(j)) > tsmaxflow(j,jj)) qstr(jst,tspltjb(j)) = tsmaxflow(j,jj)
          end do

      ! Do Withdrawals next
        else if (tspltcntr(j) == '      WD') then
          do jj=1,nouts(j)
            qall = qall + qwd(jstsplt(j,jj))                                        ! sum up all the flows
          end do
          do jj=1,nouts(j)                                                          ! set the flows and honor the maximum flow
            jwd = jstsplt(j,jj)
            qwd(jwd) = qwdfrac(jwd) * qall
            if (tsmaxflow(j,jj) > 0.0 .and. qwd(jwd) > tsmaxflow(j,jj)) qwd(jwd) = tsmaxflow(j,jj)
          end do
        end if
      end if
    end do
  end if

! Output some results.
  if (jday.ge.nxtstr) then
    nxtstr = nxtstr+tfrqtmp
    ifile=1949
    do jb=1,nbr
      if (nstr(jb) > 0) then
        ifile=ifile+1
        write (ifile,'(f10.4,<nstr(jb)>f10.2,<nstr(jb)>f10.2,<nstr(jb)>f10.2)') jday,(tavg(i,jb),i=1,nstr(jb)),(qstr(i,jb),i=1,nstr(jb)),(estr(i,jb),i=1,nstr(jb))
      end if
    end do
    if (nwd > 0) then
      ifile=ifile+1
      write (ifile,'(f10.4,<nwd>f10.2,<nwd>f10.2,<nwd>f10.2)') jday,(tavgw(i),i=1,nwd),(qwd(i),i=1,nwd),(ewd(i),i=1,nwd)
    end if

  ! computing reservoir volume and volume below 'tempcrit'
    volmc=0.0
    volm=0.0
    DO JW=1,NWB
      KT = KTWB(JW)
      DO JB=BS(JW),BE(JW)
        DO I=cus(jb),ds(jb)
          volm(jw) = volm(jw) +BH2(KT,I)*DLX(I)
          DO K=kt+1,kb(i)
            volm(jw) = volm(jw)+BH(K,I)*DLX(I)
          END DO
          do kk=1,tempn
            if(t2(kt,i).le.tempcrit(jw,kk))volmc(jw,kk) = volmc(jw,kk)+BH2(KT,I)*DLX(I)
            DO K=kt+1,kb(i)
              if(t2(k,i).le.tempcrit(jw,kk))volmc(jw,kk) = volmc(jw,kk)+BH(K,I)*DLX(I)
            END DO
          end do
        end do
      end do

      ifile=ifile+1
      write(ifile,5315)jday,volm(jw),(volmc(jw,kk), kk=1,tempn)
5315  format(f8.2,100(g12.4,g12.4))
    end do
  end if

! Check elevations and status of temperature control towers.
  if (tempc == '      ON' .and. jday .ge. nxttcd) then

  ! Update the temperature targets
    DO J=1,NUMTEMPC
      IF (DYNSEL(J) == '      ON') THEN
        DO WHILE (JDAY >= NXSEL(J))
          tctemp(j) = TEMP2(J)
          READ (SELD(J),'(1000F8.0)') NXSEL(J),TEMP2(J)
        END DO
      END IF
    END DO

    do j=1,numtempc

    ! Structures
      if (tcntr(j) == '      ST') then
        js = tcjs(j)                                 ! set structure index
        jb = tcjb(j)                                 ! set branch index
        DO JW=1,NWB                                  ! set waterbody index
          IF (JB >= BS(JW) .AND. JB <= BE(JW)) EXIT
        END DO

        if (tciseg(j) .eq. 0) then
          tcomp = tavg(js,jb)               !cb 9/8/06
        else if (tciseg(j) < 0) then
          tcomp = twdo(abs(tciseg(j)))      ! sw 11/26/10
        else

        ! Check to see if the monitoring segment tciseg is in the same branch and waterbody as the structure
          DO JJB=1,NBR
            IF (tciseg(j) >= US(JJB) .AND. tciseg(j) <= DS(JJB)) exit
          end do
          DO JJW=1,NWB
            IF (JjB >= BS(JJW) .AND. JJB <= BE(JJW)) EXIT
          END DO

          IF (tcklay(j) < 0) THEN
            K = INT(ABS(tcklay(j)))
          ELSE
            DO K=KTWB(JJW),KB(tciseg(j))
              IF (DEPTHB(K,tciseg(j)) > tcklay(j)) EXIT
            END DO
            K = MIN(K,KB(tciseg(j)))
          END IF
          tcomp = t2(k,tciseg(j))
        end if
        if (tcyearly(j) == '     OFF') then
          daytest = jday
        else
          daytest = real(jdayg) + jday - int(jday)
        end if
        if (daytest >= tctsrt(j) .and. daytest < tctend(j)) then
          estr(js,jb) = tcelev(j,ncountc(js,jb))                  ! initialize the structure elevation

          if (tcomp > tctemp(j) .and. tcnelev(j) > ncountc(js,jb)) then
          ! making sure that the next lower structure for a particular 'j' is found
            do nj=ncountc(js,jb)+1,tcnelev(j)
              if (tcelev(j,nj) < estr(js,jb)) then
                ncountc(js,jb) = nj
                estr(js,jb)    = tcelev(j,ncountc(js,jb))
                exit
              end if
            end do

          else if (tcomp < tctemp(j) .and. ncountc(js,jb) .gt. 1) then
          ! to prevent this happening at each time it checks it and hence oscillating back and forth - check the temp at the upper outlet also
            if (tciseg(j) > 0) then
              if (jb .eq. jjb) then
                wsel = elws(ds(jb)) - sina(jb) * dlx(ds(jb)) * 0.5                        ! compute water-surface elevation !SR 03/24/13
                do ks=ktwb(jw),kb(ds(jb))
                ! if (depthb(ks,tciseg(j)) > tcelev(j,ncountc(js,jb)-1)) exit             !??can't be right-- SR 03/24/13
                  if (wsel - depthb(ks,tciseg(j)) < tcelev(j,ncountc(js,jb)-1)) exit      !SR 03/24/13
                end do
                ks   = min(ks,kb(tciseg(j)))
                tmod = t2(ks,ds(jb))
              else
                tmod = t2(k,tciseg(j))
              end if
              if (tmod < tctemp(j) .and. tcelev(j,ncountc(js,jb)-1) < elws(ds(jb))) then
              ! making sure that the next upper structure for a particular 'j' is found
                do nj=ncountc(js,jb)-1,1,-1
                  if (tcelev(j,nj) > estr(js,jb)) then
                    ncountc(js,jb) = nj
                    estr(js,jb)    = tcelev(j,ncountc(js,jb))
                    exit
                  end if
                end do
              end if

            else if (tciseg(j) .eq. 0) then
            ! calculate the estimated outflow temperature at higher ports when tcomp < tctemp(j),
            ! and move up if higher port still meets criteria - this doesn't happen when tciseg < 0
              do nj=1,ncountc(js,jb)-1
                id = ds(jb)
                kt = ktwb(jw)
                call downstream_withdrawal_estimate(js,tempest,tcelev(j,nj))
                if (tempest < tctemp(j) .and. tcelev(j,nj) < elws(ds(jb))) then
                  ncountc(js,jb) = nj
                  estr(js,jb)    = tcelev(j,ncountc(js,jb))
                  exit
                end if
              end do
            end if
          end if
          if (tcelevcon(j) == '      ON' .and. tcnelev(j) > ncountc(js,jb) .and. estr(js,jb) > elws(ds(jb))) then
            ncountc(js,jb) = ncountc(js,jb)+1
            estr(js,jb)    = tcelev(j,ncountc(js,jb))
          end if
        end if

    ! Withdrawals
      else if (tcntr(j) == '      WD') then
        jwd = tcjs(j)
        if (tciseg(j) .eq. 0) then
        ! tcomp = tout(jb)
          tcomp = tavgw(tcjs(j))   !cb 9/8/06
        else if (tciseg(j) < 0) then
          tcomp = twdo(abs(tciseg(j)))
        else

        ! checking to see if the monitoring segment tciseg is in the same branch and water body as the withdrawal
          DO JJB=1,NBR
            IF (tciseg(j) >= US(JJB) .AND. tciseg(j) <= DS(JJB)) exit
          end do
          DO JJW=1,NWB
            IF (JjB >= BS(JJW) .AND. JJB <= BE(JJW)) EXIT
          END DO

          IF (tcklay(j) < 0) THEN
            K = INT(ABS(tcklay(j)))
          ELSE
            DO K=KTWB(JJW),KB(tciseg(j))
              IF (DEPTHB(K,tciseg(j)) > tcklay(j)) EXIT
            END DO
            K = MIN(K,KB(tciseg(j)))
          END IF
          tcomp = t2(k,tciseg(j))
        end if
        if (tcyearly(j) == '     OFF') then
          daytest = jday
        else
          daytest = real(jdayg) + jday - int(jday)
        end if
        if (daytest >= tctsrt(j) .and. daytest < tctend(j)) then
          ewd(jwd) = tcelev(j,ncountcw(jwd))                  ! initialize the withdrawal elevation

          if (tcomp > tctemp(j) .and. tcnelev(j) > ncountcw(jwd)) then
          ! making sure that the next lower structure for a particular 'j' is found
            do nj=ncountcw(jwd)+1,tcnelev(j)
              if (tcelev(j,nj) < ewd(jwd)) then
                ncountcw(jwd) = nj
                ewd(jwd)      = tcelev(j,ncountcw(jwd))
                exit
              end if
            end do

          else if (tcomp < tctemp(j) .and. ncountcw(jwd) .gt. 1) then
          ! to prevent this happening at each time it checks it and hence oscillating back and forth - check the temp at the upper outlet also
            if (tciseg(j) > 0) then
              tmod = t2(k,tciseg(j))
              if (tmod < tctemp(j) .and. tcelev(j,ncountcw(jwd)-1) < elws(iwd(jwd))) then
              ! making sure that the next upper structure for a particular 'j' is found
                do nj=ncountcw(jwd)-1,1,-1
                  if (tcelev(j,nj) > ewd(jwd)) then
                    ncountcw(jwd) = nj
                    ewd(jwd)      = tcelev(j,ncountcw(jwd))
                    exit
                  end if
                end do
              end if

            else if (tciseg(j) == 0) then
            ! calculate estimated outflow temperature at higher ports when tcomp < tctemp(j),
            ! and move up if higher port still meets criteria
              I = MAX(CUS(JBWD(JWD)),IWD(JWD))
              DO JJB=1,NBR
                IF (I >= US(JJB) .AND. I <= DS(JJB)) exit
              end do
              DO JJW=1,NWB
                IF (JJB >= BS(JJW) .AND. JJB <= BE(JJW)) EXIT
              END DO
              kt = ktwb(jjw)
              do nj=1,ncountcw(jwd)-1
                call lateral_withdrawal_estimate(jwd,tempest,tcelev(j,nj))
                if (tempest < tctemp(j) .and. tcelev(j,nj) < elws(iwd(jwd))) then
                  ncountcw(jwd) = nj
                  ewd(jwd)      = tcelev(j,ncountcw(jwd))
                  exit
                end if
              end do
            end if
          end if
          if (tcelevcon(j) == '      ON' .and. tcnelev(j) > ncountcw(jwd) .and. ewd(jwd) > elws(iwd(jwd))) then
            ncountcw(jwd) = ncountcw(jwd)+1
            ewd(jwd)      = tcelev(j,ncountcw(jwd))
          end if
        end if
      end if
    end do

    nxttcd = nxttcd + tcdfreq
  end if
return


ENTRY DEALLOCATE_SELECTIVE
  DEAllocate (tcnelev,tcjb,tcjs, tcelev,tctemp,tctend,tctsrt,ncountc,tciseg,tcklay,tcelevcon,elcontspl)
  DEAllocate (tspltjb,tspltt,nouts,jstsplt,kstrsplt,tcyearly, tcntr,tspltcntr)
  DEallocate (volm,ncountcw,qwdfrac,qstrfrac)
  DEallocate (tempcrit,volmc,DYNSEL,SELD,NXSEL,TEMP2,TSYEARLY,TSTEND,TSTSRT)
  deallocate (tsdepth, tstype, tsminfrac, tsprior, tsminhead, tsmaxhead, tsmaxflow, no_flow)
  deallocate (tsdynsel, tsseld, nxtssel, tstemp2, ewdsav, estrsav, share_flow, wd_active, str_active)
  deallocate (nout0, nout1, nout2, minfrac1, minfrac2, maxfrac1, maxfrac2, splt2t, splt2e)
RETURN

End Subroutine Selective


!***********************************************************************************************************************************
!**                                                S E T _ F L O W _ F R A C T I O N S                                            **
!***********************************************************************************************************************************
                                                                              ! Entire routine added/modified by S. Rounds, 06/26/13
Subroutine Set_Flow_Fractions
  Use Selective1                                                              ! This routine sets the flow fractions, and then
                                                                              ! redistributes flow to other outlets in the same
  integer :: n, nj, nexcess, j, jb, jst, jwd, j2pref                          ! group when one or more outlets exceed their maximum
  real    :: excess_frac, addfrac                                             ! flow rates.  Excess flow that cannot be accommodated
Return                                                                        ! within each group will be discarded.


Entry Set_Flow_Fracs(j, jb, j2pref)
  excess_frac = 0.0                                                           ! Excess flow above maximum release rates
  nexcess     = 0                                                             ! Number of group 1 outlets exceeding maximum rates

! Set and rebalance release fractions for group 1 structures
  if (tspltcntr(j) == '      ST') then
    do n=1,ng1                                                                ! Find maxed-out outlets in group 1; set flows to max
      jst = jstsplt(j,nout1(n))
      qstrfrac(jst,jb) = minfrac1(n) + (qfrac1 - sum_minfrac1) / ng1
      if (qstrfrac(jst,jb) > maxfrac1(n)) then
        nexcess = nexcess + 1
        excess_frac = excess_frac + qstrfrac(jst,jb) - maxfrac1(n)
        qstrfrac(jst,jb) = maxfrac1(n)
      end if
    end do
    if (excess_frac > 0.0 .and. ng1 - nexcess > 0) then                       ! Redistribute excess flow to other outlets in group
      do nj=1,ng1                                                             ! Iterative process, in case others get maxed-out
        if (ng1 == nexcess .or. excess_frac <= 0.00001) exit
        addfrac = excess_frac / (ng1 - nexcess)
        do n=1,ng1
          jst = jstsplt(j,nout1(n))
          if (maxfrac1(n) - qstrfrac(jst,jb) > 0.00001) then
            if (qstrfrac(jst,jb) + addfrac > maxfrac1(n)) then
              nexcess = nexcess + 1
              excess_frac = excess_frac - (maxfrac1(n) - qstrfrac(jst,jb))
              qstrfrac(jst,jb) = maxfrac1(n)
            else
              excess_frac = excess_frac - addfrac
              qstrfrac(jst,jb) = qstrfrac(jst,jb) + addfrac
            end if
          end if
        end do
      end do
    end if

! Set and rebalance release fractions for group 1 withdrawals
  else
    do n=1,ng1                                                                ! Find maxed-out outlets in group 1; set flows to max
      jwd = jstsplt(j,nout1(n))
      qwdfrac(jwd) = minfrac1(n) + (qfrac1 - sum_minfrac1) / ng1
      if (qwdfrac(jwd) > maxfrac1(n)) then
        nexcess = nexcess + 1
        excess_frac = excess_frac + qwdfrac(jwd) - maxfrac1(n)
        qwdfrac(jwd) = maxfrac1(n)
      end if
    end do
    if (excess_frac > 0.0 .and. ng1 - nexcess > 0) then                       ! Redistribute excess flow to other outlets in group
      do nj=1,ng1                                                             ! Iterative process, in case others get maxed-out
        if (ng1 == nexcess .or. excess_frac <= 0.00001) exit
        addfrac = excess_frac / (ng1 - nexcess)
        do n=1,ng1
          jwd = jstsplt(j,nout1(n))
          if (maxfrac1(n) - qwdfrac(jwd) > 0.00001) then
            if (qwdfrac(jwd) + addfrac > maxfrac1(n)) then
              nexcess = nexcess + 1
              excess_frac = excess_frac - (maxfrac1(n) - qwdfrac(jwd))
              qwdfrac(jwd) = maxfrac1(n)
            else
              excess_frac = excess_frac - addfrac
              qwdfrac(jwd) = qwdfrac(jwd) + addfrac
            end if
          end if
        end do
      end do
    end if
  end if

Entry Set_Flow_Fracs2(j, jb, j2pref)                                          ! Separate entry just for group 2 outlets

  if (j2pref == 0) j2pref = 1                                                 ! Preferred outlet number, if not sharing
  excess_frac = 0.0                                                           ! Excess flow above maximum release rates
  nexcess     = 0                                                             ! Number of group 2 outlets exceeding maximum rates

! Set and rebalance release fractions for group 2 structures
  if (tspltcntr(j) == '      ST') then
    do n=1,ng2                                                                ! Find maxed-out outlets in group 2; set flows to max
      jst = jstsplt(j,nout2(n))
      if (.not. share_flow(j) .and. ng2 > 1) then                             ! Direct flow to preferred outlet if not shared
        if (n == j2pref) then
          qstrfrac(jst,jb) = qfrac2 - sum_minfrac2 + minfrac2(n)
        else
          qstrfrac(jst,jb) = minfrac2(n)
        end if
      else
        qstrfrac(jst,jb) = minfrac2(n) + (qfrac2 - sum_minfrac2) / ng2
      end if
      if (qstrfrac(jst,jb) > maxfrac2(n)) then
        nexcess = nexcess + 1
        excess_frac = excess_frac + qstrfrac(jst,jb) - maxfrac2(n)
        qstrfrac(jst,jb) = maxfrac2(n)
      end if
    end do
    if (excess_frac > 0.0 .and. ng2 - nexcess > 0) then                       ! Redistribute excess flow to other outlets in group
      do nj=1,ng2                                                             ! Iterative process, in case others get maxed-out
        if (ng2 == nexcess .or. excess_frac <= 0.00001) exit
        addfrac = excess_frac / (ng2 - nexcess)
        do n=1,ng2
          jst = jstsplt(j,nout2(n))
          if (maxfrac2(n) - qstrfrac(jst,jb) > 0.00001) then
            if (qstrfrac(jst,jb) + addfrac > maxfrac2(n)) then
              nexcess = nexcess + 1
              excess_frac = excess_frac - (maxfrac2(n) - qstrfrac(jst,jb))
              qstrfrac(jst,jb) = maxfrac2(n)
            else
              excess_frac = excess_frac - addfrac
              qstrfrac(jst,jb) = qstrfrac(jst,jb) + addfrac
            end if
          end if
        end do
      end do
    end if

! Set and rebalance release fractions for group 2 withdrawals
  else if (tspltcntr(j) == '      WD') then
    do n=1,ng2                                                                ! Find maxed-out outlets in group 2; set flows to max
      jwd = jstsplt(j,nout2(n))
      if (.not. share_flow(j) .and. ng2 > 1) then                             ! Direct flow to preferred outlet if not shared
        if (n == j2pref) then
          qwdfrac(jwd) = qfrac2 - sum_minfrac2 + minfrac2(n)
        else
          qwdfrac(jwd) = minfrac2(n)
        end if
      else
        qwdfrac(jwd) = minfrac2(n) + (qfrac2 - sum_minfrac2) / ng2
      end if
      if (qwdfrac(jwd) > maxfrac2(n)) then
        nexcess = nexcess + 1
        excess_frac = excess_frac + qwdfrac(jwd) - maxfrac2(n)
        qwdfrac(jwd) = maxfrac2(n)
      end if
    end do
    if (excess_frac > 0.0 .and. ng2 - nexcess > 0) then                       ! Redistribute excess flow to other outlets in group
      do nj=1,ng2                                                             ! Iterative process, in case others get maxed-out
        if (ng2 == nexcess .or. excess_frac <= 0.00001) exit
        addfrac = excess_frac / (ng2 - nexcess)
        do n=1,ng2
          jwd = jstsplt(j,nout2(n))
          if (maxfrac2(n) - qwdfrac(jwd) > 0.00001) then
            if (qwdfrac(jwd) + addfrac > maxfrac2(n)) then
              nexcess = nexcess + 1
              excess_frac = excess_frac - (maxfrac2(n) - qwdfrac(jwd))
              qwdfrac(jwd) = maxfrac2(n)
            else
              excess_frac = excess_frac - addfrac
              qwdfrac(jwd) = qwdfrac(jwd) + addfrac
            end if
          end if
        end do
      end do
    end if
  end if
Return

End Subroutine Set_Flow_Fractions
